#include "scheduler.h"
#include "testcases.h"

#include <stdlib.h>  // for srand(), rand() and RAND_MAX
#include <time.h>
#include <math.h>  // for sqrt(), log() and sin()
#include <string>
#include <fstream>
// draw number from normal random distribution
double normal(const double &mean, const double &std)
{
  static const double pii=3.1415927;
  static const double r_max=RAND_MAX+1;
  return std*sqrt(-2*log((rand()+1)/r_max))*sin(2*pii*rand()/r_max)+mean;
} 

#define DISTRIBUTIONS 3
// vraca slucajni broj u odabranoj raspodjeli
// izlaz: broj od 0.01-1
double random(int distribution = 1)
{
	double pt;
	switch(distribution)
	{	case 1:	// jednolika raspodjela
			//pt = (rand()%1000 + 10) / 1000.;
			pt = rand()%100 + 1;
			pt /= 100;
			//if(pt<0.01) pt = 0.01;
		break;
		case 2:	// neka vrlo jednostavna normalna raspodjela
		{	pt = (int) normal(51,20);
			pt /= 100;
			if(pt>1) pt = 1;
			if(pt<0.01) pt = 0.01;
		}
		break;
		case 3:	// moja: kvazi bimodalna
		{	double B = 0.0046511627;
			double A = 2*B, C = 3*B;
			pt = (rand()%1001) / 1000.;
			if(pt<=0.093)
				pt = (pt + A) / A;
			else if(pt<=0.316)
				pt = (pt - 0.093 + 11*C) / C;
			else if(pt<=0.465)
				pt = (pt - 0.316 + 27*A) / A;
			else if(pt<=0.535)
				pt = (pt - 0.465 + 43*B) / B;
			else if(pt<=0.684)
				pt = (pt - 0.535 + 58*A) / A;
			else if(pt<=0.907)
				pt = (pt - 0.684 + 74*C) / C;
			else
				pt = (pt - 0.907 + 90*A) / A;
			pt /= 100;
		}
		break;
		default: CHECK(0);
	}
	return pt;
}


// cita parametre i pravi matrice sa test nizovima
void Scheduler::MakeSets(char input[])
{
	ReadPar p;
	double d_niz[4][2000],pt,distribution[DISTRIBUTIONS];
	char duration[256],deadline[256],sp[256],weight[256],ready[256],speed[256];
	int sets,max_jobs,max_machines=1;
	int i,j,k,l,m;
	bool taillard = false;
	Matrica M,Minv,MachineIndex;

	srand( (unsigned)time( NULL ) );

	if(!p.OpenFile(input)) {
		cout << "Nema ulazne datoteke! (" << input << ")\n";
		return;
	}
	if(p.ReadParameter("single",INTEGER,&i))
		environment = SINGLE;
	else if(p.ReadParameter("uniform",INTEGER,&i))
		environment = UNIFORM;
	else if(p.ReadParameter("unrelated",INTEGER,&i))
		environment = UNRELATED;
	else if(p.ReadParameter("jobshop",INTEGER,&i))
		environment = JOBSHOP;
	else CHECKMSG(0,"Nije definirano okruzenje!");
	if(p.ReadParameter("taillard",INTEGER,&i))	// oznaka - trajanja i raspored generirati po Taillardu
		taillard = true;
	p.ReadParameter("sets",INTEGER,&sets);
	p.ReadParameter("max_jobs",INTEGER,&max_jobs);
	p.ReadParameter("max_machines",INTEGER,&max_machines);
	p.ReadParameter("max_length",INTEGER,&max_length);
	p.ReadParameter("duration",STRING,&duration);
	p.ReadParameter("deadline",STRING,&deadline);
	p.ReadParameter("SP",STRING,&sp);
	p.ReadParameter("machine_file",STRING,&speed);
	if(p.ReadParameter("weight_T",STRING,&weight))
		weights = 1;
	else weights = 0;
	if(p.ReadParameter("ready",STRING,&ready))
		dynamic = 1;
	else
		dynamic = 0;
	if(p.ReadParameter("distribution",DOUBLE,distribution,DISTRIBUTIONS))
	{	// racunamo kumulativne vjerojatnosti
		double sum = 0;
		for(i=0; i<DISTRIBUTIONS; i++)
			sum += distribution[i];
		double ratio = 1/sum;
		sum = 0;
		for(i=0; i<DISTRIBUTIONS; i++)
		{	distribution[i] = sum + distribution[i]*ratio;
			sum = distribution[i];
		}
	}
	else distribution[0] = 1;

	R.Init(sets);
	T.Init(sets);
	N.Init(sets);
	SP.Init(sets);
	SD.Init(sets);
	Machines.Init(sets);
	M.Init(sets);
	Minv.Init(sets);
	MachineIndex.Init(max_machines);
	Speed.Init(sets,max_machines);
	if(environment == UNRELATED)
		Duration.Init(sets, max_jobs * max_machines);	// komotna verzija
	else if(environment == JOBSHOP)
		Duration.Init(sets, max_jobs * max_machines);
	else
		Duration.Init(sets,max_jobs);
	Deadline.Init(sets,max_jobs);
	Weight.Init(sets,max_jobs);
	Ready.Init(sets,max_jobs);
	p.ReadParameter("R.",DOUBLE,&d_niz[0][0],sets);
	p.ReadParameter("T.",DOUBLE,&d_niz[1][0],sets);
	p.ReadParameter("jobs",DOUBLE,&d_niz[2][0],sets);
	p.ReadParameter("machines",DOUBLE,&d_niz[3][0],sets);
	for(i=0; i<sets; i++)
	{	R[i][0] = d_niz[0][i];
		T[i][0] = d_niz[1][i];
		N[i][0] = d_niz[2][i];
		Machines[i][0] = d_niz[3][i];
	}
	// punjenje matrica
	// brzine strojeva za jednoliko okruzenje
	if(environment == UNIFORM)
	{	for(i=0; i<sets; i++)
		{	M[i][0] = 0; Minv[i][0] = 0;
			for(j=0; j<max_machines; j++)
				if(j<Machines[i][0])
				{	Speed[i][j] = pow( rand()%90/100. + 0.1, 1);	// ako je vrijeme obrade p_jk = p_j * s_k
					//Speed[i][j] = 1/spd;	// ako uzmemo vrijeme obrade po definiciji: p_jk = p_j / s_k 
					M[i][0] += 1/Speed[i][j];
					Minv[i][0] += Speed[i][j];
				}
				else
					Speed[i][j] = 0;
		}
		Speed.Save(speed);
	}
	// trajanje poslova
	if(environment == SINGLE || environment == UNIFORM)
	{	for(i=0; i<sets; i++)
		{	SP.Set(i,0);
			for(j=0; j<max_jobs; j++)
			{	if(j<N.Get(i))
				{	pt = random();
					int dist = DISTRIBUTIONS;
					for(k=1; k<DISTRIBUTIONS; k++)
						if(pt<=distribution[k-1])
						{	dist = k; break;	}
					Duration[i][j] = (int) (random(dist) * 100);
					SP[i][0] += Duration[i][j];
				}
				else
					Duration[i][j] = 0;
			}
			if(environment == UNIFORM)	// prilagodba ukupnog trajanja - ovdje to moramo aproksimirati
				SP[i][0] /= M[i][0];	// podijelimo sa ukupom brzinom strojeva
				//SP[i][0] /= Machines[i][0];
		}
	}
	else if(environment == UNRELATED)
	{	for(i=0; i<sets; i++)
		{	SP[i][0] = 0;
			m = (int) Machines[i][0];
			for(j=0; j<N[i][0]; j++)
			{	for(k=0; k<m; k++)
				{	pt = random();
					int dist = DISTRIBUTIONS;
					for(int D=1; D<DISTRIBUTIONS; D++)
						if(pt<=distribution[D-1])
						{	dist = D; break;	}
					Duration[i][j*m + k] = (int) (random(dist) * 100);
					SP[i][0] += Duration[i][j*m + k];
				}
			}
			SP[i][0] /= pow(Machines[i][0], 2);	// procjena sume trajanja poslova
			// dijelimo sa kvadratom jer smo za svaki posao ubrajali vrijeme na svakom stroju
			for(j=(int) (N[i][0]*m); j<max_jobs*max_machines; j++)
				Duration[i][j] = 0;
		}
	}
	else if(environment == JOBSHOP)
	{	if(taillard)
			::MakeTaillardTests(Duration,N,SP);
		else
		for(i=0; i<sets; i++)
		{	SP[i][0] = 0;
			// u jednostavnijoj varijanti uzimamo da je broj opracija svakog posla jednak broju strojeva
			m = (int) Machines[i][0];
			for(j=0; j<N[i][0]; j++)	// poslovi
			{	for(k=0; k<m; k++)		// operacije
				{	pt = random();
					int dist = DISTRIBUTIONS;
					for(int D=1; D<DISTRIBUTIONS; D++)
						if(pt<=distribution[D-1])
						{	dist = D; break;	}
					Duration[i][j*m + k] = (int) (random(dist) * 100);
					SP[i][0] += Duration[i][j*m + k];
					MachineIndex[k][0] = k;
				}
				for(k=0; k<m; k++)		// strojevi za operacije
				{	int swap = (int) (random() * (m-1));
					double pom = MachineIndex[k][0];
					MachineIndex[k][0] = MachineIndex[swap][0];
					MachineIndex[swap][0] = pom;
				}
				for(k=0; k<m; k++)		// zapisivanje indeksa stroja (tisucice)
				{	if(k == 2)	// hm...
					{	Duration[i][j*m + k] *= 1.5;
						if(Duration[i][j*m + k] > 100)
							Duration[i][j*m + k] = 100;
					}
					Duration[i][j*m + k] += (int) MachineIndex[k][0] * 1000;
				}
			}
			SP[i][0] /= Machines[i][0];	// procjena ukupnog trajanja
			for(j=(int) (N[i][0]*m); j<max_jobs*max_machines; j++)
				Duration[i][j] = 0;
		}
	}

	// potom ready times (ako nije definirano, onda 0)
	for(i=0; i<sets; i++)
	{	for(j=0; j<max_jobs; j++)
			if(j<N.Get(i) && dynamic == 1)	// e sad problematicno
			{	// recimo max dolazak je na 0.5 ukupnog trajanja?
				// a raspodjela jednolika?
				k = rand()%static_cast<int>(SP.Get(i)*0.5);	
				Ready[i][j] = static_cast<double>(k);
			}
			else
				Ready[i][j] = 0;
	}
	// pa onda deadline; odnosno, 'due dates' preciznije
	for(i=0; i<sets; i++)
	{	SD.Set(i,0);
		for(j=0; j<max_jobs; j++)
		{	if(j<N.Get(i))
			{	// jednolika raspodjela za deadline
				if(dynamic == 0)
				{	k = (int) (SP.Get(i) * (1 - T.Get(i) - R.Get(i)/2));	// donja
					l = (int) (SP.Get(i) * (1 - T.Get(i) + R.Get(i)/2));	// i gornja granica
				}
				else	// dinamicki - mnozi se sa razlikom ukupnog trajanja i dolaska posla!
				{	k = (int) ( (SP.Get(i)-Ready[i][j]) * (1 - T.Get(i) - R.Get(i)/2));
					l = (int) ( (SP.Get(i)-Ready[i][j]) * (1 - T.Get(i) + R.Get(i)/2));
				}
				k = k + (rand()%1000)*(l-k)/1000;	// izmedju
				if(k<0) k = 0;
				Deadline[i][j] = k + Ready.Get(i,j);	// deadline je k + vrijeme dolaska!
				SD[i][0] += Deadline[i][j];
			}
			else
				Deadline[i][j] = 0;
		}
	}
	// pa tezine
	for(i=0; i<sets; i++)
		for(j=0; j<max_jobs; j++)
			if(j<N.Get(i))
				Weight[i][j] = rand()%100/100. + 0.01;
			else
				Weight[i][j] = 0;
	Duration.Save(duration);
	Deadline.Save(deadline);
	if(dynamic == 1)
		Ready.Save(ready);
	SP.Save(sp);
	if(weights == 1)
		Weight.Save(weight);
	p.CloseFile();
}

// cita parametre i samo ispituje heuristike
void Scheduler::Test(char input[], char outfile[])
{
	ReadPar p;
	Matrica AlgValues[20];
	string algoritmi[20];
	double d_niz[4][2000];
	char duration[256],deadline[256],sp[256],weight[256],ready[256],cons[256],speed[256];
	unsigned int sets,max_jobs,max_machines;
	unsigned int i,j,total_jobs,algoritam;

	if(!p.OpenFile(input)) {
		cout << "Nema ulazne datoteke! (" << input << ")\n";
		return;
	}
	if(p.ReadParameter("single",INTEGER,&i)) {
		environment = SINGLE;
		cout << "Okruzenje: jedan stroj\n";
	}
	else if(p.ReadParameter("uniform",INTEGER,&i)) {
		environment = UNIFORM;
		cout << "Okruzenje: paralelni jednoliki strojevi\n";
	}
	else if(p.ReadParameter("unrelated",INTEGER,&i)) {
		environment = UNRELATED;
		cout << "Okruzenje: paralelni nesrodni strojevi\n";
	}
	else if(p.ReadParameter("jobshop",INTEGER,&i)) {
		environment = JOBSHOP;
		cout << "Okruzenje: proizvolja obrada (job shop), kriterij Cmax\n";
	}
	else CHECKMSG(0,"Nije definirano okruzenje!");
	p.ReadParameter("sets",INTEGER,&sets);
	p.ReadParameter("max_jobs",INTEGER,&max_jobs);
	p.ReadParameter("max_machines",INTEGER,&max_machines);
	p.ReadParameter("max_length",INTEGER,&max_length);
	p.ReadParameter("duration",STRING,&duration);
	p.ReadParameter("deadline",STRING,&deadline);
	p.ReadParameter("SP",STRING,&sp);
	p.ReadParameter("machine_file",STRING,&speed);
	if(p.ReadParameter("weight_T",STRING,&weight))
		weights = 1;
	else weights = 0;
	if(p.ReadParameter("ready",STRING,&ready))
		dynamic = 1;
	else
		dynamic = 0;
	if(p.ReadParameter("constraints",STRING,&cons))
		constraints = 1;
	else
		constraints = 0;
	if(p.ReadParameter("setup",DOUBLE,&sqs_faktor))
		setup = 1;
	else
		setup = 0;

	algoritam = 0;
	R.Init(sets);
	T.Init(sets);
	N.Init(sets);
	SP.Init(sets);
	SD.Init(sets);
	Machines.Init(sets);
	Speed.Init(sets,max_jobs);
	Ready.Reset(sets,max_jobs);	// nule ako nije dinamicko
	Duration.Init(sets,max_jobs);
	Deadline.Init(sets,max_jobs);
	Weight.Init(sets,max_jobs);
	Constraints.Init(sets,max_jobs);
	p.ReadParameter("R.",DOUBLE,&d_niz[0][0],sets);
	p.ReadParameter("T.",DOUBLE,&d_niz[1][0],sets);
	p.ReadParameter("jobs",DOUBLE,&d_niz[2][0],sets);
	p.ReadParameter("machines",DOUBLE,&d_niz[3][0],sets);
	total_jobs = 0;
	for(i=0; i<sets; i++)
	{	R[i][0] = d_niz[0][i];
		T[i][0] = d_niz[1][i];
		N[i][0] = d_niz[2][i];
		total_jobs += (int) d_niz[2][i];
		Machines[i][0] = d_niz[3][i];
	}
	Duration.Load(duration);
	Deadline.Load(deadline);
	if(weights == 1)
		Weight.Load(weight);
	SP.Load(sp);
	if(dynamic == 1)
		Ready.Load(ready);
	if(constraints == 1)
		Constraints.Load(cons);
	// racunamo sumu deadline-a
	for(i=0; i<sets; i++)
	{	SD.Set(i,0);
		for(j=0; j<(unsigned int)N.Get(i); j++)
			SD[i][0] += Deadline[i][j];
	}
	p.CloseFile();

	if(environment == SINGLE)
	{
		// gledamo li OR rjesenja
		ifstream ORFile;
		or_solutions = 0;
		if(sORFile != "")
		{	ORFile.open(sORFile.c_str());
			if(ORFile.is_open())
			{	or_solutions = 1;
				ORFile.close();
			}
		}
		
		MakeSchedule(&Scheduler::ValueEDD);
		printf("EDD:\t %g\n",Evaluate());
		Values.Save("rezultat_EDD.txt");
		BestValues = Values;
		algoritmi[algoritam] = "EDD";
		AlgValues[algoritam++] = Values;

		MakeSchedule(&Scheduler::ValueWSPT);
		printf("WSPT:\t %g\n",Evaluate());
		Values.Save("rezultat_WSPT.txt");
		UpdateBestValues();
		algoritmi[algoritam] = "WSPT";
		AlgValues[algoritam++] = Values;

		MakeSchedule(&Scheduler::ValueRM);
		printf("R&M:\t %g\n",Evaluate());
		Values.Save("rezultat_RM.txt");
		//Schedule = Schedule + 1;
		Schedule.Save("raspored_RM.txt");
		UpdateBestValues();
		algoritmi[algoritam] = "RM";
		AlgValues[algoritam++] = Values;

		MakeSchedule(&Scheduler::ValueXD);
		printf("X-D R&M:\t %g\n",Evaluate());
		Values.Save("rezultat_XD.txt");
		UpdateBestValues();
		algoritmi[algoritam] = "XD";
		AlgValues[algoritam++] = Values;

		MakeSchedule(&Scheduler::ValueMON);
		printf("MON:\t %g\n",Evaluate());
		Values.Save("rezultat_MON.txt");
		UpdateBestValues();
		algoritmi[algoritam] = "MON";
		AlgValues[algoritam++] = Values;

		if(!constraints && !setup && !dynamic)
		{	MakeScheduleNwt_H1(Schedule,N,Duration,Deadline,SP,Weight,Ready,Constraints);
			printf("Nwt heuristika 1:\t %g\n",Evaluate());
			Values.Save("rezultat_Nwt.txt");
			UpdateBestValues();
			algoritmi[algoritam] = "Nwt_1";
			AlgValues[algoritam++] = Values;

			if(or_solutions)
			{	algoritmi[algoritam] = "OR best";
				AlgValues[algoritam] = Values;
				Values.Load(sORFile.c_str());
				AlgValues[algoritam].SetColumn(0, Values);
				Values = AlgValues[algoritam];
				UpdateBestValues();
				algoritam++;
			}
		}

		if(constraints)
		{	Schedule.Load("raspored_Sidney.txt");
			Schedule = Schedule + (-1);
			printf("Sidney:\t %g\n",Evaluate());
			Values.Save("rezultat_Sidney.txt");
			UpdateBestValues();
			algoritmi[algoritam] = "Sidney";
			AlgValues[algoritam++] = Values;

			MakeSchedule(&Scheduler::ValueHL);
			printf("HLF:\t %g\n",Evaluate());
			Values.Save("rezultat_HLF.txt");
			UpdateBestValues();
			algoritmi[algoritam] = "HLF";
			AlgValues[algoritam++] = Values;

			MakeSchedule(&Scheduler::ValueLNS);
			printf("LNS:\t %g\n",Evaluate());
			Values.Save("rezultat_LNS.txt");
			UpdateBestValues();
			algoritmi[algoritam] = "LNS";
			AlgValues[algoritam++] = Values;
		}

		if(setup)
		{	MakeSchedule(&Scheduler::ValueATCS);
			printf("ATCS:\t %g\n",Evaluate());
			Values.Save("rezultat_ATCS.txt");
			UpdateBestValues();
			algoritmi[algoritam] = "ATCS";
			AlgValues[algoritam++] = Values;

			MakeSchedule(&Scheduler::ValueMakespan);
			printf("SDS-BD makespan:\t %g\n",Evaluate());
			Values.Save("rezultat_SQSBD_makespan.txt");
			UpdateBestValues();
			algoritmi[algoritam] = "makespan";
			AlgValues[algoritam++] = Values;
		}
	}

	if(environment == UNIFORM)
	{	Speed.Load(speed);

		MakeScheduleMultiple(&Scheduler::ValueXD);
		printf("X-D RM: %g\n",EvaluateMultiple());
		Values.Save("rezultat_XD.txt");
		MakeScheduleMultiple(&Scheduler::ValueMON);
		printf("MON: %g\n",EvaluateMultiple());
		Values.Save("rezultat_MON.txt");
		MakeScheduleMultiple(&Scheduler::ValueWSPT);
		printf("WSPT: %g\n",EvaluateMultiple());
		Values.Save("rezultat_SPT.txt");
		MakeScheduleMultiple(&Scheduler::ValueEDD);
		printf("EDD: %g\n",EvaluateMultiple());
		Values.Save("rezultat_EDD.txt");
		MakeScheduleMultiple(&Scheduler::ValueRM);
		printf("RM: %g\n",EvaluateMultiple());
		Values.Save("rezultat_RM.txt");
		MakeScheduleMultiple(&Scheduler::ValueATCS);
		printf("ATCS: %g\n",EvaluateMultiple());
		Values.Save("rezultat_ATCS.txt");
		MakeScheduleMultiple(&Scheduler::ValueLNS);
		printf("LNS: %g\n",EvaluateMultiple());
		Values.Save("rezultat_LNS.txt");
		MakeScheduleMultiple(&Scheduler::ValueHL);
		printf("HLF: %g\n",EvaluateMultiple());
		Values.Save("rezultat_HLF.txt");
		MakeScheduleMultiple(&Scheduler::ValueMakespan);
		printf("Makespan: %g\n",EvaluateMultiple());
		Values.Save("rezultat_makespan.txt");
	}

	if(environment == UNRELATED)
	{	MakeScheduleMinMin();
		printf("Min-min: %g\n", EvaluateMultiple());
		Schedule.Save("raspored_minmin.txt");
		Values.Save("rezultat_minmin.txt");
	}
	if(environment == JOBSHOP)
	{
	/*	for(int i=0; i<2; i++)
		{	weights = i;
			if(weights)
				printf("\ntezine...\n");
			MakeScheduleJobShop(&Scheduler::JSValueWSPT);
			printf("SPT: %g\n",EvaluateJobShop());
			if(weights)
				Values.Save("rezultat_WSPT.txt");
			else
			{	Values.Save("rezultat_SPT.txt");
				Schedule.Save("raspored_SPT.txt");
			}
			MakeScheduleJobShop(&Scheduler::JSValueWSPT_TWKR);
			printf("SPT/TWKR: %g\n",EvaluateJobShop());
			if(weights)
				Values.Save("rezultat_WSPT_TWKR.txt");
			else
				Values.Save("rezultat_SPT_TWKR.txt");
			MakeScheduleJobShop(&Scheduler::JSValueCOVERT);
			printf("COVERT: %g\n",EvaluateJobShop());
			if(weights)
				Values.Save("rezultat_WCOVERT.txt");
			else
				Values.Save("rezultat_COVERT.txt");
			MakeScheduleJobShop(&Scheduler::JSValueSLACK_TWKR);
			printf("SLACK/TWKR: %g\n",EvaluateJobShop());
			if(weights)
				Values.Save("rezultat_WSLACK_TWKR.txt");
			else
				Values.Save("rezultat_SLACK_TWKR.txt");
			MakeScheduleJobShop(&Scheduler::JSValueRM);
			printf("RM: %g\n",EvaluateJobShop());
			if(weights)
				Values.Save("rezultat_WRM.txt");
			else
				Values.Save("rezultat_RM.txt");
		}*/

		MakeScheduleJobShop(&Scheduler::JSValueWSPT);
		printf("WSPT: %g\n",EvaluateJobShop());
		Values.Save("rezultat_WSPT.txt");
		// inicijalno najbolja rjesenja
		BestValues = Values;
		algoritmi[algoritam] = "WSPT\t";
		AlgValues[algoritam++] = Values;

		MakeScheduleJobShop(&Scheduler::JSValueWSPT_TWKR);
		printf("WSPT/TWKR: %g\n",EvaluateJobShop());
		Values.Save("rezultat_WSPT_TWKR.txt");
		UpdateBestValues();
		algoritmi[algoritam] = "WSPT/TWKR";
		AlgValues[algoritam++] = Values;

		MakeScheduleJobShop(&Scheduler::JSValueCOVERT);
		printf("COVERT: %g\n",EvaluateJobShop());
		Values.Save("rezultat_WCOVERT.txt");
		UpdateBestValues();
		algoritmi[algoritam] = "COVERT\t";
		AlgValues[algoritam++] = Values;

		MakeScheduleJobShop(&Scheduler::JSValueSLACK_TWKR);
		printf("SLACK/TWKR: %g\n",EvaluateJobShop());
		Values.Save("rezultat_WSLACK_TWKR.txt");
		UpdateBestValues();
		algoritmi[algoritam] = "SLACK/TWKR";
		AlgValues[algoritam++] = Values;

		MakeScheduleJobShop(&Scheduler::JSValueRM);
		printf("RM: %g\n",EvaluateJobShop());
		Values.Save("rezultat_WRM.txt");
		UpdateBestValues();
		algoritmi[algoritam] = "RM\t";
		AlgValues[algoritam++] = Values;

		MakeScheduleJobShop(&Scheduler::JSValueEDD);
		printf("EDD: %g\n",EvaluateJobShop());
		Values.Save("rezultat_EDD.txt");
		UpdateBestValues();
		algoritmi[algoritam] = "EDD\t";
		AlgValues[algoritam++] = Values;

		MakeScheduleJobShop(&Scheduler::JSValueWTWKR);
		printf("WTWKR: %g\n",EvaluateJobShop());
		Values.Save("rezultat_WTWKR.txt");
		UpdateBestValues();
		algoritmi[algoritam] = "WTWKR\t";
		AlgValues[algoritam++] = Values;
	}

	// za GP
	Schedule.Load("raspored_GP.txt");
	if(environment == SINGLE)
		printf("GP:\t %g\n", Evaluate());
	else if(environment == UNIFORM || environment == UNRELATED)
		printf("GP:\t %g\n", EvaluateMultiple());
	else if(environment == JOBSHOP)
		printf("GP:\t %g\n", EvaluateJobShop());
	Values.Save("rezultat_fGP.txt");
	UpdateBestValues();
	algoritmi[algoritam] = "GP";
	AlgValues[algoritam++] = Values;

	// ispis GP rezultata u outfile
	ofstream out;
	if(outfile)
	{	out.open(outfile, ios_base::app);
		for(int crit = 0; crit<nCriteria; crit++)
			out << "\t" << Values[Values.GetRow() - 1][crit];
	}

	// statistike - postotak dominacije
	cout << "\nDominacija: (kriteriji Twt, Uwt, Fwt, Cmax)\n";
	if(environment == SINGLE || environment == JOBSHOP)
	{	for(unsigned int alg = 0; alg<algoritam; alg++)
		{	int best[nCriteria] = {0, 0, 0, 0};
			for(int set = 0; set<AlgValues[alg].GetRow() - 1; set++)
				for(int crit = 0; crit < nCriteria; crit++)
				{	if(AlgValues[alg][set][crit] == BestValues[set][crit]
						// ovo ima utjecaja u release verziji!!!
						|| fabs(AlgValues[alg][set][crit] - BestValues[set][crit]) < 0.0000000001)
							best[crit]++;
				}

			double result[nCriteria];
			for(int crit = 0; crit < nCriteria; crit++)
				result[crit] = 100. * best[crit] / (AlgValues[alg].GetRow() - 1);

			if(algoritmi[alg] == "GP" && out.is_open())
			{	for(int crit = 0; crit < nCriteria; crit++)
					out << "\t" << result[crit];
				out.close();
			}
			cout << algoritmi[alg] << "\t";
			for(int crit = 0; crit < nCriteria; crit++)
				printf(" %5.1f\t", result[crit]);
			cout << endl;
		}
	}

}