// funkcije za evaluaciju
#include<vector>
using namespace std;

#include"scheduler.h"

#define MIN(x,y) (x<y ? x : y)
#define MAX(x,y) (x>y ? x : y)

// evaluacija gotovog rasporeda za JOBSHOP okruzenje
double Scheduler::EvaluateJobShop(void)
{
	double time,duration,weights,setuptime,durations;
	double Cmax, Cmax_set, Fwt, Fwt_set, F, F_set, T, T_set, Twt, Twt_set, U, U_set, Uwt, Uwt_set, L;
	int i, j, sets, max_jobs, jobs, job, set, schedule;
	int max_machines, machine, machine_index, machines, job_operation, machine_operation;
	Matrica MachineReady, LastJob, JobReady;
	Matrica MachineScheduled, OperationsScheduled, JobTime, MachineTime;

	Cmax = Fwt = F = T = Twt = U = Uwt = durations = 0;
	sets = Duration.GetRow();
	max_jobs = Deadline.GetCol();
	max_machines = Duration.GetCol() / max_jobs;
	Values.Reset(sets+1,7);
	MachineReady.Init(max_machines);
	Precedence.Reset(max_jobs,max_jobs+1);
	Level.Reset(max_jobs);
	LastJob.Init(max_machines);	// koji je bio zadnji posao na stroju
	Durations.Init(max_jobs, max_machines);		// matrica trajanja operacija za jedan skup
	MachineIndex.Init(max_jobs, max_machines);	// matrica indeksa strojeva po operacijama za jedan skup
	OperationsScheduled.Init(max_jobs);			// koliko je operacija pojedinog posla rasporedjeno
	MachineScheduled.Init(max_machines);		// koliko je operacija na pojedinom stroju rasporedjeno
	JobReady.Init(max_jobs);					// kada ce zavrsiti zadnja rasporedjena operacija pojedinog posla
	JobTime.Init(max_jobs, max_machines);
	MachineTime.Init(max_machines, max_jobs);

	for(set=0; set<sets; set++)
	{	jobs = (int) N.Get(set);
		machines = (int) Machines[set][0];
		time = durations = 0;
		for(j=0; j<machines; j++)
		{	MachineScheduled[j][0] = 0;
			MachineReady[j][0] = 0;	// pocetno vrijeme raspolozivosti strojeva
			LastJob[j][0] = jobs;	// pocetni prethodni posao
		}
		for(i=0; i<jobs; i++)
		{	JobReady[i][0] = Ready[set][i];
			OperationsScheduled[i][0] = 0;
			for(j=0; j<machines; j++)
			{	duration = Duration[set][i*machines + j];
				machine = (int) (duration) / 1000;	// indeks stroja
				duration = (int) (duration) % 1000;		// trajanje operacije
				Durations[i][j] = duration;
				durations += duration;	// ukupno trajanje
				MachineIndex[i][j] = machine;
			}
		}
		durations /= jobs;	// prosjecno trajanje svih poslova
		if(setup)
			MakeSetup(set);
		// staticki i dinamicki dolasci poslova
		for(schedule=0; schedule<jobs*machines; schedule++)
		{	for(machine=0; machine<machines; machine++)
			{	machine_operation = (int) MachineScheduled[machine][0];	// koja operacija je na redu na stroju
				if(machine_operation == jobs) continue;		// ovaj stroj je gotov
				job = (int) Schedule[set][machine*jobs + machine_operation];	// od kojeg posla
				job_operation = (int) OperationsScheduled[job][0];	// koja operacija je na redu za taj posao
				machine_index = (int) MachineIndex[job][job_operation];	// na kojem stroju
				if(machine != machine_index) continue;	// ako se ne podudaraju, nista
				// inace ide ovaj posao na ovaj stroj
				time = MAX(JobReady[job][0], MachineReady[machine][0]);
				if(setup)
				{	setuptime = Setup[(int)LastJob[machine][0]][job];
					LastJob[machine][0] = job;
				}
				else setuptime = 0;
				MachineReady[machine][0] = time + setuptime + Durations[job][job_operation];
				JobReady[job][0] = MachineReady[machine][0];
				MachineScheduled[machine][0] += 1;
				OperationsScheduled[job][0] += 1;
			}
		}
		// kriteriji
		weights = 0;
		Cmax_set = F_set = Fwt_set = T_set = Twt_set = U_set = Uwt_set = 0;
		for(j=0; j<jobs; j++)
		{	weights += Weight[set][j];
			if(Cmax_set < JobReady[j][0])
				Cmax_set = JobReady[j][0];
			F_set += JobReady[j][0] - Ready[set][j];
			Fwt_set += (JobReady[j][0] - Ready[set][j]) * Weight[set][j];
			if(JobReady[j][0] > Deadline[set][j])
			{	U_set += 1;
				Uwt_set += Weight[set][j];
				L = JobReady[j][0] - Deadline[set][j];
				T_set += L;
				Twt_set += Weight[set][j] * L;
			}
		}
		weights /= jobs;
		Cmax_set /= (jobs*durations);
		F_set /= jobs;
		Fwt_set /= jobs*weights*durations;
		T_set /= jobs;
		Twt_set /= jobs*weights*durations;
		U_set /= jobs;
		Uwt_set /= jobs*weights;
		Values[set][0] = Cmax_set;
		Values[set][1] = F_set;
		Values[set][2] = Fwt_set;
		Values[set][3] = T_set;
		Values[set][4] = Twt_set;
		Values[set][5] = U_set;
		Values[set][6] = Uwt_set;
		Cmax += Cmax_set;
		F += F_set;
		Fwt += Fwt_set;
		T += T_set;
		Twt += Twt_set;
		U += U_set;
		Uwt += Uwt_set;
	}
	Values[sets][0] = Cmax;
	Values[sets][1] = F;
	Values[sets][2] = Fwt;
	Values[sets][3] = T;
	Values[sets][4] = Twt;
	Values[sets][5] = U;
	Values[sets][6] = Uwt;
	return Cmax;
}


// generira matricu sequence dependant setup (trajanja postavljanja)
// za posao na pocetku koristi se zadnji (jobs-ti) redak matrice
void Scheduler::MakeSetup(int set)
{
	int i,j;
	int jobs = (int) N[set][0], machines = (int) Machines[set][0];
	Savg = 0;
	Setup.Init(jobs+1,jobs);
	if(environment == JOBSHOP)
	{	srand(set);
		for(i=0; i<jobs; i++)
		{	Setup[jobs][i] = (int) ((rand()%max_length+1) * sqs_faktor);	// polazno trajanje postavljanja
			for(j=0; j<=i; j++)
			{	Setup[i][j] = (int) ((rand()%max_length+1) * sqs_faktor);
				//Setup[i][j] = (Setup[i][j] + middle/2) / 2;
				Setup[j][i] = (int) ((rand()%max_length+1) * sqs_faktor);
				//Setup[j][i] = (Setup[j][i] + middle/2) / 2;
				Savg += Setup[i][j] + Setup[j][i];
			}
		}
	}
	else
		for(i=0; i<jobs; i++)
		{	Setup[jobs][i] = Duration[set][(i+1) % jobs];	// polazno trajanje postavljanja
			for(j=0; j<=i; j++)
			{	Setup[i][j] = ceil( fabs( Duration[set][i] - Duration[set][j] ) * sqs_faktor);
				Setup[j][i] = ceil( fabs( Duration[set][(i+1) % jobs] - Duration[set][(j+1) % jobs] ) * sqs_faktor);
				Savg += Setup[i][j] + Setup[j][i];
			}
		}
	Savg /= jobs*jobs - jobs;	// globalno prosjecno trajanje postavljanja
}

// rekurzivno racunanje 'level' vrijednosti svakog posla
// ima smisla samo u problemima s ogranicenjima
double Scheduler::NodeLevel(int set,int node)
{	double value,level,duration;
	int succ,i,next;
	if(environment == SINGLE)
		duration = Duration[set][node];
	else if(environment == UNIFORM)
		duration = Duration[set][node]/Mavg;
	else CHECK(0);
	if(Level[node][0] > -1)	// ako je vec izracunato
		return Level[node][0];
	if(Precedence[node][1] == 0)	// ako nema sljedbenika
		return duration;
	succ = (int)Precedence[node][1];	// koliko sljedbenika
	next = (int)Precedence[node][2];	// prvi sljedbenik
	level = NodeLevel(set,next) + duration;	// moj level je level sljedbenika + moje trajanje
	for(i=1; i<succ; i++)
	{	next = (int)Precedence[node][i+2];
		value = NodeLevel(set,next) + duration;
		if(value > level)
			level = value;
	}
	return level;
}
/* cita matricu ogranicenja za sve skupove i generira citljiviju matricu za zadani skup
- 0. stupac: broj mojih prethodnika
- 1. stupac: broj mojih sljedbenika
- sljedeci stupci: indeksi mojih sljedbenika
- jobs-ti stupac: max vrijeme zavrsetka mojih prethodnika (dodano za multimachine okruzenje)
posao moze krenuti ako mu je broj prethodnika stavljen na nulu (svi pokrenuti) i ako je trenutno vrijeme
vece od max vremena zavrsetka
*/
void Scheduler::ReadConstraints(int set)
{
	int i,j,prethodnika,prethodnik,pom;
	unsigned int podatak;
	int jobs = (int) N[set][0];
	Precedence.Init(jobs,jobs+1);
	Level.Init(jobs);
	// prvo se matrica ocisti!
	for(i=0; i<jobs; i++)
	{	Level[i][0] = -1;	// oznaka da nije izracunato
		for(j=0; j<jobs; j++)
			Precedence[i][j] = 0;
	}
	for(i=0; i<jobs; i++)
	{	podatak = (unsigned int) Constraints[set][i];
		prethodnik = 1;	// koji po redu posao iza mene
		prethodnika = 0;
		while(podatak != 0)
		{	if(podatak%2 != 0)
			{	prethodnika++;	// povecam broj svojih prethodnika
				pom = (int) Precedence[i-prethodnik][1] + 1;
				Precedence[i-prethodnik][pom+1] = i;
				Precedence[i-prethodnik][1] = pom;	// i broj sljedbenika od moga prethodnika
			}
			prethodnik++;
			podatak /= 2;
		}
		Precedence[i][0] = prethodnika;
	}
	for(i=0; i<jobs; i++)	// na kraju i matricu levela cvorova
		Level[i][0] = NodeLevel(set,i);
}

// ocjenjuje generirani raspored po 'svim' kriterijima i zapisuje u matricu Values
// radi za SINGLE okruzenje
// promjene: ukljucena normizacija rezultata po skupovima, dijeljenjem sa brojem poslova i srednjom tezinom
/*
Promjene za usporedbu s web benchmarkom (OR library): u OR primjerima rezultati su izrazeni s tezinama
poslova [1, 10], pa se 'nase' vrijednosti [0, 1] mnoze s 10. Takodjer se u OR primjerima rjesenja ne 
normiraju po broju poslova i srednjoj tezini.
*/
double Scheduler::Evaluate(void)
{
	int set,index,sets,job,clock,jobs,last_job;
	double late,total_late,Twt,Twt_set,Nwt,Nwt_set,Cmax,Fwt,Fwt_set,Ewt,Ewt_set,weights,durations;

	sets = Schedule.GetRow();
	Values.Init(sets+1,4);	// matrica sa rezultatima za svaki set
	Twt = Nwt = Cmax = Fwt = Ewt = 0;
	total_late = 0;
	for(set=0; set<sets; set++)
	{	clock = 0; jobs = (int) N.Get(set);
		last_job = jobs; weights = durations = 0;
		Twt_set = Nwt_set = Fwt_set = Ewt_set = 0;
		if(setup)
			MakeSetup(set);
		for(job=0; job<N.Get(set); job++)
		{	index = (int) Schedule.Get(set,job);
			if(dynamic == 1 && clock<Ready.Get(set,index))	// ako posao jos nije stigao
				clock = (int) Ready.Get(set,index);	// onda postavimo vrijeme na njegov dolazak
				// moze i drugacije, ali ovako je ocuvan slijed poslova u rasporedu
			clock += (int) Duration.Get(set, index);
			if(setup)
				clock += (int) Setup[last_job][index];
			last_job = index;
			if(clock > (int) Deadline.Get(set,index))
			{	late = clock - (int) Deadline.Get(set,index);
				total_late += 1;
				Nwt_set += Weight.Get(set,index);

				if(or_solutions)
					late *= Weight.Get(set,index)*10;	// za usporedbu sa web benchmarkom
				else
					late *= Weight.Get(set,index);
				Twt_set += late;
			}
			if(dynamic)
				Fwt_set += clock - Ready[set][index];
			else
				Fwt_set += clock - 0;
			weights += Weight[set][job];
			durations += Duration[set][job];
		}
		weights /= jobs;
		durations /= jobs;

		if(or_solutions)
			Twt += Twt_set;	// za usporedbu sa web benchmarkom
		else
			Twt += Twt_set/(jobs*weights*durations);
		Nwt += Nwt_set/(jobs*weights);
		Fwt += Fwt_set/(jobs*weights*durations);
		Cmax += clock/(jobs*durations);

		if(or_solutions)
			Values[set][0] = Twt_set;	// za usporedbu sa web benchmarkom
		else
			Values[set][0] = Twt_set/(jobs*weights*durations);
		Values[set][1] = Nwt_set/(jobs*weights);
		Values[set][2] = Fwt_set/(jobs*weights*durations);
		Values[set][3] = clock/jobs;
	}
	Values[sets][0] = Twt;
	Values[sets][1] = Nwt;
	Values[sets][2] = Fwt;
	Values[sets][3] = Cmax;
	return Twt;
}

// Uzima matricu Values i u matricu BestValues zapisuje bolja rjesenja (po svim kriterijima)
void Scheduler::UpdateBestValues(void)
{
	int sets = Values.GetRow() - 1;
	int pom = 0, dum = 0;
	for(int set = 0; set<sets; set++)
		for(int criteria = 0; criteria<nCriteria; criteria++)
		{
			if(criteria == 0 && Values[set][criteria] <= BestValues[set][criteria])
				pom++;

			if(criteria == 0 && Values[set][criteria] < BestValues[set][criteria])
				dum++;

			if(Values[set][criteria] < BestValues[set][criteria])
			{	BestValues[set][criteria] = Values[set][criteria];
				//pom++;
			}
		}
	sets--;
}


// ocjenjuje generirani raspored po 'svim' kriterijima i zapisuje u matricu Values
// radi za UNIFORM i UNRELATED okruzenja
// dodano normiranje rezultata po skupovima
// POTREBNO: ispraviti ocjenjivanje s ogranicenjima!
double Scheduler::EvaluateMultiple(void)
{
	int i,set,index,sets,job,jobs,last_job,machines,machine;
	double clock,late,total_late,Twt,Twt_set,Nwt,Nwt_set,Cmax,Fwt,Fwt_set,Ewt,Ewt_set,Lmax,Lmax_set,weights,durations;
	Matrica MachineTime, LastJob;

	sets = Schedule.GetRow();
	Values.Init(sets+1,5);	// matrica sa rezultatima za svaki set
	Twt = Nwt = Cmax = Fwt = Ewt = Lmax = 0;
	total_late = 0;
	for(set=0; set<sets; set++)
	{	machines = (int) Machines[set][0];
		MachineTime.Reset(machines);	// vremena za svaki stroj
		LastJob.Init(machines);
		clock = 0; jobs = (int) N.Get(set);
		last_job = jobs; weights = durations = 0;
		Twt_set = Nwt_set = Fwt_set = Ewt_set = Lmax_set = 0;
		if(setup)
		{	MakeSetup(set);
			for(i=0; i<machines; i++)
				LastJob[i][0] = jobs;
		}
		for(job=0; job<N.Get(set); job++)
		{	index = (int) Schedule.Get(set,job);
			machine = index / 1000;	// indeks stroja
			index = index % 1000;	// indeks posla
			if(dynamic == 1 && MachineTime[machine][0]<Ready[set][index])	// ako posao jos nije stigao
				MachineTime[machine][0] = Ready[set][index];	// onda postavimo vrijeme na njegov dolazak
				// moze i drugacije, ali ovako je ocuvan slijed poslova u rasporedu
			if(environment == UNIFORM)
				MachineTime[machine][0] += Duration[set][index] * Speed[set][machine];
			else if(environment == UNRELATED)
				MachineTime[machine][0] += Duration[set][index*machines + machine];
			if(setup)
				MachineTime[machine][0] += Setup[(int)LastJob[machine][0]][index];
			LastJob[machine][0] = index;
			if(MachineTime[machine][0] > Deadline[set][index])
			{	late = MachineTime[machine][0] - Deadline[set][index];
				if(late > Lmax_set)
					Lmax_set = late;
				total_late += 1;
				Nwt_set += Weight[set][index];
				late *= Weight[set][index];
				Twt_set += late;
			}
			if(dynamic)
				Fwt_set += MachineTime[machine][0] - Ready[set][index];
			else
				Fwt_set += MachineTime[machine][0] - 0;
			weights += Weight[set][job];
			durations += Duration[set][job];
		}
		weights /= jobs;
		durations = SP[set][0] / jobs;
		Twt += Twt_set/(jobs*weights*durations);
		Nwt += Nwt_set/(jobs*weights);
		//Fwt += Fwt_set/(jobs*durations);	// netezinsko protjecanje
		Fwt += Fwt_set/(jobs*weights*durations);
		Lmax += Lmax_set/(durations);
		clock = MachineTime[0][0];
		for(i=1; i<machines; i++)	// nadjemo Cmax
			if(MachineTime[i][0] > clock)
				clock = MachineTime[i][0];
		Cmax += clock/(jobs*durations);
		Values[set][0] = Twt_set/(jobs*weights*durations);
		Values[set][1] = Nwt_set/(jobs*weights);
		//Values[set][2] = Fwt_set/(jobs*durations);	// netezinsko protjecanje
		Values[set][2] = Fwt_set/(jobs*weights*durations);
		Values[set][3] = clock/(jobs*durations);
		Values[set][4] = Lmax_set/(durations);
	}
	Values[sets][0] = Twt;
	Values[sets][1] = Nwt;
	Values[sets][2] = Fwt;
	Values[sets][3] = Cmax;
	Values[sets][4] = Lmax;
	return Twt;
}

double Scheduler::ValueWSPT(int set, int job, int last_job, double time, int machine)
{
	double rez, speed = Speed[set][machine],pt;
	pt = Duration[set][job];
	pt += POS(Ready[set][job] - time);	// za dinamicko okruzenje
	if(environment == SINGLE)
	{	rez = Weight[set][job] / pt;
		if(setup)
			rez -= Setup[last_job][job]/Pav/pt;
	}
	else if(environment == UNIFORM)	// moze biti jednako kao za SINGLE!
	{	rez = Weight[set][job] / (pt / M);
		if(setup)
			rez -= Setup[last_job][job]/(Pav/M)/(pt*speed);
	}
	return rez;
}

double Scheduler::ValueRM(int set, int job, int last_job, double time, int machine)
{
	static double pt,w,dd,sp,slack,rez=0, speed = Speed[set][machine];
	pt = Duration.Get(set,job);
	pt += POS(Ready[set][job] - time);	// za dinamicko okruzenje
	w = Weight.Get(set,job);
	dd = Deadline.Get(set,job);
	sp = SP.Get(set);
	if(environment == SINGLE)
	{	slack = POS(dd - pt - time);
		//if(slack<0) slack = 0;
		rez = (w / pt) * exp(-slack / (2. * Pav));
		if(setup)
			rez -= Setup[last_job][job]/(Pav)/(Duration[set][job]);
	}
	else if(environment == UNIFORM)
	{	slack = dd - pt*speed - time;
		if(slack<0) slack = 0;
		rez = (w / (pt / M)) * exp( -slack / (Pav / M));
		if(setup)
			rez -= Setup[last_job][job]/(Pav/M)/(Duration[set][job]*speed);
	}
	return rez;
}

double Scheduler::ValueXD(int set, int job, int last_job, double time, int machine)
{
	static double pt,w,dd,sp,slack,ar,p,rez, speed = Speed[set][machine];
	pt = Duration.Get(set,job);
	w = Weight.Get(set,job);
	dd = Deadline.Get(set,job);
	sp = SP.Get(set);
	ar = POS(Ready[set][job] - time);
	if(environment == SINGLE)
	{	slack = dd - pt - time;
		if(slack<0) slack = 0;
		p = (w / pt) * exp(-slack / (2. * Pav));
		rez = p * (1 - (1.3 + 1)*ar/Pav);	// stvarno ne znam koliko staviti 2. clan u zagradi
		// lokalno ispitivanje: 30 otprilike najbolji za ove probleme
		if(setup)
			rez -= Setup[last_job][job]/(Pav)/(Duration[set][job]);
	}
	else if(environment == UNIFORM)
	{	slack = dd - pt*speed - time;
		if(slack<0) slack = 0;
		p = (w / (pt / M)) * exp( -slack / (Pav/M));
		rez = p * (1 - (1.3 + 30)*ar/ (Pav/M) );
		if(setup)
			rez -= Setup[last_job][job]/(Pav/M)/(Duration[set][job]*speed);
	}
	return rez;
}

// isti rezultat za UNIFORM okruzenje
double Scheduler::ValueMON(int set, int job, int last_job, double time, int machine)
{
	double pt,w,dd,sp,rez=0;
	pt = Duration.Get(set,job);
	pt += POS(Ready[set][job] - time);	// za dinamicko okruzenje
	w = Weight.Get(set,job);
	dd = Deadline.Get(set,job);
	sp = SP.Get(set);
	if(setup)
		rez -= Setup[last_job][job]/(Pav)/Duration[set][job];
	return rez + w * (1. - dd / sp) / (pt);
}

double Scheduler::ValueMakespan(int set, int job, int last_job, double time, int machine)
{
	double rez, speed = Speed[set][machine];
	if(environment == SINGLE)
	{	if(setup)
			return -Setup[last_job][job]/Pav/Duration[set][job];
		else
			return 1;
	}
	else if(environment == UNIFORM)	// LPT heuristika
	{	rez = Duration[set][job] / M;
		if(setup)
			rez -= Setup[last_job][job]/(Pav/M)/(Duration[set][job]*speed);
	}
	return rez;
}

// isti rezultat za UNIFORM okruzenje
double Scheduler::ValueATCS(int set, int job, int last_job, double time, int machine)
{
	static double pt,w,dd,sp,slack,stp;
	pt = Duration[set][job];
	pt += POS(Ready[set][job] - time);	// za dinamicko okruzenje
	w = Weight[set][job];
	dd = Deadline[set][job];
	sp = SP[set][0];
	slack = dd - pt - time;
	if(slack<0)
		slack = 0;
	if(setup)
		stp = Setup[last_job][job];
	else stp = 0;
	return (w / pt) * exp(-slack / (ATCS1*PTavg) - stp / (ATCS2*Savg));
}


double Scheduler::ValueEDD(int set, int job, int last_job, double time, int machine)
{
	double rez = -Deadline[set][job];
	if(setup)
		rez -= Setup[last_job][job]/Pav/Duration[set][job];
	return rez;
}

double Scheduler::ValueLNS(int set, int job, int last_job, double time, int machine)
{	// Largest Number of Successors - minimizira makespan
	return Precedence[job][1];
}

double Scheduler::ValueHL(int set, int job, int last_job, double time, int machine)
{	// Highest Level First - minimizira makespan
	return Level[job][0];
}
// SINGLE okruzenje
// radi proizvoljni raspored uz navodjenje funkcije koja racuna prioritet posla
// prilagodjeno za dinamicke uvjete, ogranicenja u rasporedu i proizvoljna vremena postavljanja
void Scheduler::MakeSchedule(double (Scheduler::*Fja)(int set, int job, int last_job, double time, int machine) )
{
	double najkrace,time,prvi,davg,T,R,eta,dmax,dmin;
	int j,sets,max_jobs,jobs,remaining,job,chosen,set,sljedbenik,last_job=-1;
	Matrica Priority, Scheduled;

	sets = Duration.GetRow();
	max_jobs = Duration.GetCol();
	Priority.Init(max_jobs);
	Schedule.Init(sets, max_jobs);
	Scheduled.Init(max_jobs);
	Precedence.Reset(max_jobs,max_jobs);
	Level.Reset(max_jobs);

	for(set=0; set<sets; set++)
	{	remaining = jobs = (int) N.Get(set);
		time = 0; Pav = 0;
		last_job = jobs;	// pocetno trajanje postavljanja
		for(j=0; j<N.Get(set); j++)
		{	Pav += Duration.data[set][j];
			Scheduled.data[j][0] = 0;
		}
		Pav /= jobs;
		PTavg = Pav;	//glob varijabla; prosjek za sve poslove
		if(constraints)
			ReadConstraints(set);
		if(setup)
			MakeSetup(set);
		else
			Savg = 0;
		// hrpa parametara za ACTS heuristiku
			C1max = PTavg*jobs + jobs*Savg;
			eta = Savg / PTavg;
			davg = 0;
			dmax = dmin = Deadline[set][j];
			for(j=0; j<jobs; j++)
			{	davg += Deadline[set][j];
				if(Deadline[set][j] > dmax)
					dmax = Deadline[set][j];
				if(Deadline[set][j] < dmin)
					dmin = Deadline[set][j];
			}
			davg /= jobs;
			T = 1 - davg/C1max;
			R = (dmax - dmin) / C1max;
			if(R <= 0.5)
				ATCS1 = 4.5 + R;
			else
				ATCS1 = 6 - 2*R;
			ATCS2 = T / (2 * sqrt(eta));

		for(job=0; job<N.Get(set); job++)
		{	if(dynamic)
			{	for(j=0; Scheduled.data[j][0]==1 || Precedence[j][0]>0; j++) NULL; // pronadjemo prvi nerasporedjeni
				chosen = j; najkrace = Duration[set][chosen]; prvi = Ready[set][chosen];
				for(j=0; j<N.Get(set); j++)	// najblize vrijeme dolaska
				{	if(Scheduled[j][0] || Precedence[j][0]>0)
						continue;
					if(Ready[set][j] < prvi)
					{	prvi = Ready[set][j];
						najkrace = Duration[set][j];
						chosen = j;
					}
				}
				if(prvi>time)
					time = prvi;
				for(j=0; j<N.Get(set); j++)	// pronadjemo najkraci raspolozivi
				{	if(Scheduled[j][0] || Precedence[j][0]>0)
						continue;
					if(Ready[set][j]<=time && Duration[set][j]<najkrace)
					{	chosen = j;
						najkrace = Duration[set][j];
					}
				}
				Priority[chosen][0] = (this->*Fja)(set,chosen,last_job,time,NULL);
				for(j=0; j<N.Get(set); j++)	// prioritet za sve nerasporedjene
				{	if(Scheduled.data[j][0] || Precedence[j][0]>0)
						continue;
					if(dynamic && Ready[set][j] >= time + najkrace)
						continue;
					Priority[j][0] = (this->*Fja)(set,j,last_job,time,NULL);
					if(Priority[j][0] > Priority[chosen][0])
						chosen = j;
				}
			}
			else // static
			{	for(j=0; Scheduled.data[j][0]==1 || Precedence[j][0]>0; j++) NULL; // pronadjemo prvi nerasporedjeni
				chosen = j;
				Priority[chosen][0] = (this->*Fja)(set,chosen,last_job,time,NULL);
				for(j=0; j<N.Get(set); j++)	// prioritet za sve nerasporedjene
				{	if(Scheduled.data[j][0] || Precedence[j][0]>0)
						continue;
					Priority[j][0] = (this->*Fja)(set,j,last_job,time,NULL);
					if(Priority.data[j][0] > Priority.data[chosen][0])
						chosen = j;
				}
			}

			if(constraints)
				for(j=0; j<Precedence[chosen][1]; j++)	// smanjimo broj prethodnika svim sljedbenicima
				{	sljedbenik = (int) Precedence[chosen][j+2];
					Precedence[sljedbenik][0] -= 1;
				}
			Pav = (Pav*remaining - Duration.data[set][chosen]) / (remaining-1.);
			remaining--;
			Scheduled.data[chosen][0] = 1;
			Schedule.data[set][job] = chosen;
			if(setup && last_job != -1)
				time += Setup[last_job][chosen];
			last_job = chosen;
			if(dynamic && Ready[set][chosen] > time)
				time = Ready[set][chosen];
			time += Duration.data[set][chosen];
		}
	}
}

/* prva heuristika za Nwt problem (Morton str. 139)
Poslovi se rasporede u EDD a potom se neki premjestaju na kraj (oni za koje nema sanse
da budu na vrijeme)*/
void Scheduler::MakeScheduleNwt_H1(Matrica &Schedule, Matrica N, Matrica Duration, Matrica Deadline, Matrica SP, Matrica Weight, Matrica Ready, Matrica Constraints)
{
	int set,i,j,nadjeni,trenutni,pom,kraj;
	double clock,min;
	double (Scheduler::*pmf) (int set, int job, int last_job, double time, int machine);
	pmf = &Scheduler::ValueEDD;
	MakeSchedule(pmf);
	for(set = 0; set<Schedule.GetRow(); set++)
	{	kraj = (int) N.Get(set);	// broj poslova u skupu
		i = 0;
		clock = 0;
		do	// i je i-ti posao u rasporedu; 'trenutni' je indeks posla na tom mjestu
		{	trenutni = (int) Schedule.data[set][i];
			clock += Duration.data[set][trenutni];
			if(clock > Deadline.data[set][trenutni])	// da li posao kasni
			{	// ako kasni, tada trazimo onoga sa najmanjim wi/pi za sve <=i u rasporedu
				pom = (int) Schedule.data[set][0];
				min = Weight.data[set][pom] / Duration.data[set][pom];
				nadjeni = 0;
				for(j=0; j<=i; j++)
				{	pom = (int) Schedule.data[set][j];
					if(Weight.data[set][pom]/Duration.data[set][pom] < min)
					{	min = Weight.data[set][pom]/Duration.data[set][pom];
						nadjeni = j;
					}
				}
				// sad nadjenoga premjestamo na kraj rasporeda
				pom = (int) Schedule.data[set][nadjeni];
				for(j=nadjeni; j<kraj; j++)
					Schedule.data[set][j] = Schedule.data[set][j+1];
				Schedule.data[set][kraj-1] = pom;
				if(nadjeni != i)
					i--;	// ako smo maknuli nekoga ispred, ponovno gledamo trenutnoga
				kraj--;	// one premjestene vise ne gledamo (ionako su zakasnjeli)
				clock -= Duration.data[set][pom];	// smanjujemo vrijeme za trjanje premjestenoga
				// smanjujemo i za trenutnoga jer cemo ga opet provjeriti
				clock -= Duration.data[set][trenutni];
			}
			else	// ako posao ne kasni
				i++;	// gledamo sljedecega u rasporedu
		} while(i<kraj);
	}	// kraj skupa
	// konacno, kraj
}

// UNIFORM okruzenje
// radi proizvoljni raspored uz navodjenje funkcije koja racuna prioritet posla
// postupak za jednolike: cekaj prvi slobodni stroj, izracunaj prioritete i dodijeli posao stroju
// prilagodjeno za dinamicke uvjete i proizvoljna trajanja postavljanja
// ogranicenja: zasada samo staticki
void Scheduler::MakeScheduleMultiple(double (Scheduler::*Fja)(int set, int job, int last_job, double time, int machine) )
{
	double najkrace,time,prvi,davg,T,R,eta,dmax,dmin,newtime,setuptime;
	int j,sets,max_jobs,jobs,remaining,job,chosen,set,sljedbenik,last_job,max_machines,first_machine,machine;
	Matrica Priority, Scheduled, MachineReady, LastJob;

	last_job = -1;
	sets = Duration.GetRow();
	max_jobs = Duration.GetCol();
	max_machines = Speed.GetCol();
	MachineReady.Init(max_machines);
	Priority.Init(max_jobs);
	Schedule.Reset(sets, max_jobs);
	Scheduled.Init(max_jobs);
	Precedence.Reset(max_jobs,max_jobs+1);
	Level.Reset(max_jobs);
	LastJob.Init(max_machines);	// koji je bio zadnji posao na stroju

	for(set=0; set<sets; set++)
	{	remaining = jobs = (int) N.Get(set);
		time = 0; Pav = 0;
		last_job = jobs;	// za pocetno trajanje postavljanja
		for(j=0; j<N.Get(set); j++)
		{	Pav += Duration.data[set][j];
			Scheduled.data[j][0] = 0;
		}
		M = 0;
		for(j=0; j<Machines[set][0]; j++)
		{	MachineReady[j][0] = 0;	// pocetno vrijeme raspolozivosti strojeva
			LastJob[j][0] = jobs;	// pocetni prethodni posao
			M += 1/Speed[set][j];		// suma brzina svih strojeva
		}
		Mavg = M / Machines[set][0];
		Pav /= jobs;
		PTavg = Pav;	//glob varijabla; prosjek za sve poslove
		if(constraints)
			ReadConstraints(set);
		if(setup)
			MakeSetup(set);
		else
			Savg = 0;
		// hrpa parametara za ACTS heuristiku
			C1max = PTavg*jobs + jobs*Savg;
			eta = Savg / PTavg;
			davg = 0;
			dmax = dmin = Deadline[set][j];
			for(j=0; j<jobs; j++)
			{	davg += Deadline[set][j];
				if(Deadline[set][j] > dmax)
					dmax = Deadline[set][j];
				if(Deadline[set][j] < dmin)
					dmin = Deadline[set][j];
			}
			davg /= jobs;
			T = 1 - davg/C1max;
			R = (dmax - dmin) / C1max;
			if(R <= 0.5)
				ATCS1 = 4.5 + R;
			else
				ATCS1 = 6 - 2*R;
			ATCS2 = T / (2 * sqrt(eta));

		for(job=0; job<N.Get(set); job++)
		{	if(dynamic)
			{	// pronadjemo prvi nerasporedjeni posao
				for(j=0; Scheduled.data[j][0]==1 || Precedence[j][0]>0; j++) NULL;
				chosen = j; najkrace = Duration[set][chosen]; prvi = Ready[set][chosen];
				for(j=0; j<N.Get(set); j++)	// najblize vrijeme dolaska
				{	if(Scheduled[j][0] || Precedence[j][0]>0)
						continue;
					if(Ready[set][j] < prvi)
					{	prvi = Ready[set][j];
						najkrace = Duration[set][j];
						chosen = j;
					}
				}
				first_machine = 0;	// pronadjemo prvi raspolozivi stroj
				for(j=0; j<Machines[set][0]; j++)
					if(MachineReady[j][0] < MachineReady[first_machine][0])
						first_machine = j;
				// time < max(dolazak_posla, raspolozivost_stroja) ?
				if(prvi > MachineReady[first_machine][0])
					newtime = prvi;
				else
					newtime = MachineReady[first_machine][0];
				if(time < newtime)	// cekamo dok barem jedan stroj i posao nisu raspolozivi
					time = newtime;	// to je vrijeme pocetka obrade odabranog posla

				for(j=0; j<N.Get(set); j++)	// pronadjemo najkraci raspolozivi posao
				{	if(Scheduled[j][0] || Precedence[j][0]>0)
						continue;
					if(Ready[set][j]<=time && Duration[set][j]<najkrace)
					{	chosen = j;
						najkrace = Duration[set][j];
					}
				}
				Priority[chosen][0] = (this->*Fja)(set,chosen,last_job,time,first_machine);
				for(j=0; j<N.Get(set); j++)	// prioritet za sve ostale nerasporedjene
				{	if(Scheduled.data[j][0] || Precedence[j][0]>0)
						continue;
					// za ova okruzenja necemo uzimati idle time, ali posao mora biti raspoloziv
					if(dynamic && Ready[set][j] > time )
						continue;
					Priority[j][0] = (this->*Fja)(set,j,last_job,time,first_machine);
					if(Priority[j][0] > Priority[chosen][0])
						chosen = j;
				}
			}
			else // static
			{	first_machine = 0;	// pronadjemo prvi raspolozivi stroj
				for(j=0; j<Machines[set][0]; j++)
					if(MachineReady[j][0] < MachineReady[first_machine][0])
						first_machine = j;
				time = MachineReady[first_machine][0];	// to je vrijeme moguceg pocetka sljedeceg posla
				// sad trebamo pronaci minimalno vrijeme zavrsetka prethodnika za sve poslove
				// pronadjemo prvi nerasporedjeni koji nema nepokrenutih prethodnika
				for(j=0; Scheduled.data[j][0]==1 || Precedence[j][0]>0; j++) NULL;
				chosen = j;
				if(constraints)	// sada pronadjemo onoga koji moze prvi poceti
				{	for(; j<N[set][0]; j++)
						if(Scheduled[j][0]==0 && Precedence[j][0]==0 \
							&& Precedence[j][jobs] < Precedence[chosen][jobs])
							chosen = j;
					if(Precedence[chosen][jobs] > time)	// ako moramo cekati
						time = Precedence[chosen][jobs];
				}
				Priority[chosen][0] = (this->*Fja)(set,chosen,last_job,time,first_machine);
				for(j=0; j<N.Get(set); j++)	// prioritet za sve nerasporedjene
				{	if(Scheduled[j][0] || Precedence[j][0]>0 || Precedence[j][jobs]>time)
						continue;
					Priority[j][0] = (this->*Fja)(set,j,last_job,time,first_machine);
					if(Priority[j][0] > Priority[chosen][0])
						chosen = j;
				}
			}

			Pav = (Pav*remaining - Duration.data[set][chosen]) / (remaining-1.);
			remaining--;
			Scheduled[chosen][0] = 1;
			Schedule[set][job] = chosen;	// odredimo indeks posla
			// gdje izvesti odabrani posao?
			// jedno rjesenje: odabrani posao obavljamo na stroju koji ce ga najbrze zavrsiti
				//for(machine = 0; MachineReady[machine][0] > time; machine++)	NULL;
				//if(environment == UNIFORM)
				//{	// ovdje je to najbrzi raspolozivi stroj
				//	for(j=0; j<Machines[set][0]; j++)
				//		if(MachineReady[j][0] <= time && Speed[set][j] > Speed[set][machine])
				//			machine = j;
				//}
				//else CHECK(0);
			// drugo rjesenje -  po meta-algoritmu iz Mortona: na 'first_machine' (jer smo za taj stroj racunali prioritet)
			// jer neki algoritmi uzimaju u obzir i stroj prilikom racunanja prioriteta!
			machine = first_machine;
			if(setup)
				setuptime = Setup[(int) LastJob[machine][0]][chosen];	// prethodni na tom stroju
			else setuptime = 0;
			LastJob[machine][0] = chosen;
			MachineReady[machine][0] = time + setuptime + Duration.data[set][chosen] * Speed[set][machine];
			Schedule[set][job] += machine*1000;	// odredimo indeks stroja na kojemu se obavlja
			if(constraints)
				for(j=0; j<Precedence[chosen][1]; j++)	// smanjimo broj prethodnika svim sljedbenicima
				{	sljedbenik = (int) Precedence[chosen][j+2];
					Precedence[sljedbenik][0] -= 1;
					// upisemo vrijeme naseg zavrsetka za sve sljedbenike
					if(Precedence[sljedbenik][jobs] < MachineReady[machine][0])
						Precedence[sljedbenik][jobs] = MachineReady[machine][0];
				}
		}
	}
}

// min-min (i po potrebi min-max) algoritam za UNRELATED okruzenje
// 04.08.: dodana podrska za trajanja postavljanja
void Scheduler::MakeScheduleMinMin(void)
{
	int set,job,sets,machines,max_machines,max_jobs,jobs,scheduled,best_job,last_job;
	int i,machine,jobs_to_schedule,min_job,max_job,prvi,running_jobs,machine_jobs;
	double time,dur;
	Matrica Priority, Scheduled, MachineReady, SetSchedule, C;
	Matrica MachineScheduled, MachineUsed, MachineUsedTime, ScheduledOnce;

	sets = Duration.GetRow();
	max_jobs = Weight.GetCol();
	max_machines = Duration.GetCol() / max_jobs;
	Priority.Init(max_jobs);
	Schedule.Reset(sets, max_jobs);
	Precedence.Reset(max_jobs,max_jobs);
	Level.Reset(max_jobs);
	C.Init(max_jobs,max_machines);	// matrica zavrsetka svakog posla na svakom stroju
	//MinC.Init(max_jobs);	// polje indeksa strojeva najskorijeg zavrsetka za svaki posao
	vector<int> MinC(max_jobs);
	vector<int> LastJob(max_machines);	// koji je bio zadnji posao na stroju

	for(set = 0; set<sets; set++)
	{	time = 0;
		jobs = (int) N[set][0];
		machines = (int) Machines[set][0];
		Scheduled.Reset(jobs);
		MachineReady.Reset(machines);
		if(constraints)
			ReadConstraints(set);
		if(setup)
			MakeSetup(set);
		if(dynamic)	
// ako su ukljuceni dolasci poslova, to je zapravo simulacija 'dinamickog' rada, jer algoritam ne uzima 
// u obzir buducnost sustava;
// s tim da je frekvencija rasporedjivanja najbrza moguca - cim dodje neki novi posao
		{	MachineScheduled.Reset(machines);	// koliko poslova je dodijeljeno stroju
			MachineUsed.Reset(machines);		// koliko poslova je vec pokrenuto na stroju
			MachineUsedTime.Reset(machines);	// koliko vremena su potrosili pokrenuti poslovi
			SetSchedule.Reset(machines,jobs);	// koji su to poslovi
			for(i=0; i<machines; i++)			// prethodni posao za svaki stroj
				LastJob[i] = jobs;		// na pocetku je zadnji red matrice Setup
			scheduled = 0;
			time = -1;	// jer cemo gledati dolaske koji su > od time
			do
			{	// nadji prvi dolazak > od 'time' 
				for(prvi=0; Ready[set][prvi]<=time; prvi++)	NULL;
				for(i=prvi; i<jobs; i++)
					if(Ready[set][i] > time && Ready[set][i] < Ready[set][prvi])
						prvi = i;
				time = Ready[set][prvi];
				// za svaki stroj odredi koji su poslovi u medjuvremenu pokrenuti
				scheduled = 0;
				for(machine=0; machine<machines; machine++)
				{	running_jobs = (int) MachineUsed[machine][0];
					while(MachineScheduled[machine][0] > 0  &&  (MachineUsedTime[machine][0] < time && running_jobs < MachineScheduled[machine][0]))
					{	job = (int) SetSchedule[machine][running_jobs];
						Scheduled[job][0] = 1;
						if(MachineUsedTime[machine][0] < Ready[set][job])	// ako smo morali cekati na posao
							MachineUsedTime[machine][0] = Ready[set][job];
						MachineUsedTime[machine][0] += Duration[set][job*machines + machine];
						if(setup)
						{	MachineUsedTime[machine][0] += Setup[LastJob[machine]][job];
							LastJob[machine] = job;	// samo ovdje obnavljamo polje LastJob!
						}
						running_jobs++;
					}
					MachineUsed[machine][0] = MachineScheduled[machine][0] = running_jobs;
					scheduled += running_jobs;
					if(MachineUsedTime[machine][0] < time)	// ako nije bilo poslova do sada, pomaknemo vrijeme
						MachineUsedTime[machine][0] = time;
				}
				// rasporedi sve raspolozive, osim pokrenutih
				MachineReady = MachineUsedTime;
				jobs_to_schedule = 0;
				for(job = 0; job<jobs; job++)
				{	if(Scheduled[job][0]>0 || Ready[set][job]>time)
						continue;
					jobs_to_schedule++;
					//min_job = max_job = job;
					for(machine=0; machine<machines; machine++)
					{	//C[job][machine] = (MachineReady[machine][0]>time) ? MachineReady[machine][0] : time; 
						C[job][machine] = MachineReady[machine][0] + Duration[set][job*machines + machine];
						if(setup)
							C[job][machine] += Setup[LastJob[machine]][job];
					}
				}
				scheduled += jobs_to_schedule;
				ScheduledOnce.Reset(jobs);	// oznaka rasporedjenosti u ovoj iteraciji
				do	// ovo je sad min-min algoritam
				{	for(job=0; job<jobs; job++)	// nadji najbrzi stroj za svaki posao
					{	if(ScheduledOnce[job][0] > 0 || Scheduled[job][0]>0 || Ready[set][job]>time)
							continue;
						MinC[job] = 0;
						for(machine = 0; machine<machines; machine++)
							if(C[job][machine] < C[job][MinC[job]])
								MinC[job] = machine;
						min_job = max_job = job;
					}
					for(job=0; job<jobs; job++)	// nadji najskoriji zavrsetak od svih poslova
					{	if(ScheduledOnce[job][0] > 0 || Scheduled[job][0]>0 || Ready[set][job]>time)
							continue;
						// min-min
						if(C[job][MinC[job]] < C[min_job][MinC[min_job]])
							min_job = job;
						// min-max
						if(C[job][MinC[job]] > C[max_job][MinC[max_job]])
							max_job = job;
					}
					best_job = min_job;		// min-min
					//best_job = max_job;	// min-max
					// dodijeli best_job na MinC[best_job] stroj
						ScheduledOnce[best_job][0] = 1;
						machine = MinC[best_job];
						SetSchedule[machine][(int) MachineScheduled[machine][0]] = best_job;
						dur = Duration[set][best_job*machines + machine];
						if(setup)
						{	if(MachineScheduled[machine][0] > 0)
								last_job = (int) SetSchedule[machine][(int) MachineScheduled[machine][0] - 1];
							else
								last_job = jobs;
							dur += Setup[last_job][best_job];
						}
						MachineScheduled[machine][0] += 1;
						jobs_to_schedule--;

					MachineReady[machine][0] += dur;
					for(job=0; job<jobs; job++)	// obnovi vrijeme zavrsetka na tom stroju za sve poslove
					{	if(Scheduled[job][0]>0 || Ready[set][job]>time)
							continue;
						C[job][machine] += dur;
					}
				}while(jobs_to_schedule>0);
			}while(scheduled < jobs);
		}
		else	// static
		{	CHECK(0);
		}
		// sad prepisemo konacni raspored
		i = 0;
		for(machine=0; machine<machines; machine++)
		{	machine_jobs = (int) MachineScheduled[machine][0];
			for(job=0; job<machine_jobs; job++)
			{	Schedule[set][i] = machine*1000 + SetSchedule[machine][job];
				i++;
			}
		}
	}
}

// JOBSHOP okruzenje
// radi proizvoljni raspored uz navodjenje funkcije koja racuna prioritet posla
// staticki i dinamicki dolasci poslova
/* 
Za razliku od LEKIN-a, rasporedjivanje na stroju zapocet ce tek kada su raspolozivi i stroj i barem jedna 
operacija za taj stroj (zbog mogucnosti da neke operacija i zapocne i zavrsi prije neke druge op. u 
buducnosti. A u svakom slucaju mozemo kod odredjivanja prioriteta uzeti u obzir i one operacije koje jos
nisu raspolozive - bitno je da barem jedna jest.
-- Hm, LEKIN isto radi tako, iako u helpu nije bas najbolje objasnjeno...
*/
void Scheduler::MakeScheduleJobShop(double (Scheduler::*Fja)(int set, int job, int operation, double time, double ready, int last) )
{
	double time,value,duration,best,m1,m2,setuptime,najkrace,pocetak;
	bool bValueSet;
	int i, j, sets, max_jobs, jobs, job, set, last_job, best_job, schedule;
	int max_machines,machine,operation,next_machine,machine_index,machines;
	Matrica Priority, Scheduled, MachineReady, LastJob, OperationReady, JobReady;
	Matrica MachineScheduled, OperationsScheduled;

	last_job = -1;
	sets = Duration.GetRow();
	max_jobs = Deadline.GetCol();
	max_machines = Duration.GetCol() / max_jobs;
	MachineReady.Init(max_machines);
	Priority.Init(max_jobs);
	Schedule.Reset(sets, max_machines*max_jobs);	// redoslijed operacija svakog posla za svaki stroj
	Scheduled.Init(max_jobs);
	Precedence.Reset(max_jobs,max_jobs+1);
	Level.Reset(max_jobs);
	LastJob.Init(max_machines);	// koji je bio zadnji posao na stroju
	Durations.Init(max_jobs, max_machines);		// matrica trajanja operacija za jedan skup
	MachineIndex.Init(max_jobs, max_machines);	// matrica indeksa strojeva po operacijama za jedan skup
	OperationReady.Init(max_machines);			// kada je raspoloziva prva operacija za odredjeni stroj
	OperationsScheduled.Init(max_jobs);			// koliko je operacija pojedinog posla rasporedjeno
	MachineScheduled.Init(max_machines);		// koliko je operacija na pojedinom stroju rasporedjeno
	JobReady.Init(max_jobs);					// kada ce zavrsiti zadnja rasporedjena operacija pojedinog posla

	for(set=0; set<sets; set++)
	{	jobs = (int) N.Get(set);
		machines = (int) Machines[set][0];
		Pkav.Reset(machines);	// prosjecna trajanja operacija na pojedinom stroju
		time = 0; Pav = 0;
		last_job = jobs;	// za pocetno trajanje postavljanja
		for(j=0; j<machines; j++)
		{	MachineScheduled[j][0] = 0;
			MachineReady[j][0] = 0;	// pocetno vrijeme raspolozivosti strojeva
			OperationReady[j][0] = -1;	// raspolozivost operacija za strojeve tek treba odrediti
			LastJob[j][0] = jobs;	// pocetni prethodni posao
		}
		for(i=0; i<jobs; i++)
		{	JobReady[i][0] = Ready[set][i];	// Ready su nule u staticnom slucaju
			OperationsScheduled[i][0] = 0;
			for(j=0; j<machines; j++)
			{	duration = Duration[set][i*machines + j];
				machine = (int) (duration) / 1000;	// indeks stroja
				duration = (int) (duration) % 1000;		// trajanje operacije
				Durations[i][j] = duration;
				MachineIndex[i][j] = machine;
				Pkav[machine][0] += duration;
			}
			// pogledamo na kojem je stroju prva operacija
			machine = (int) MachineIndex[i][0];
			OperationReady[machine][0] = JobReady[i][0];
			// znaci taj stroj je trazen u trenutku dolaska posla (0 u statickom slucaju)
		}
		for(i=0; i<machines; i++)
			Pkav[i][0] /= jobs;		// prosjecno trajanje svih operacija na stroju
		if(setup)
			MakeSetup(set);
		// staticki i dinamicki dolasci poslova
		for(schedule = 0; schedule < jobs*machines; schedule++)
		{	// treba pronaci stroj koji ima najranije vrijeme raspolozivosti i sebe i operacije za njega
			for(machine = 0; machine<machines; machine++)
				if(OperationReady[machine][0] >= 0)	// je li stroj uopce trazen
					break;
			for(i = machine; i < machines; i++)
			{	if(OperationReady[i][0] < 0)	// jos se nije pojavila potreba za tim strojem
					continue;
				m1 = MAX(MachineReady[i][0],OperationReady[i][0]);
				m2 = MAX(MachineReady[machine][0],OperationReady[machine][0]);
				if(m1 < m2)	// ima neki stroj koji je prije raspoloziv za rasporediti
					machine = i;
			}
			time = MAX(MachineReady[machine][0], OperationReady[machine][0]);
			// pronadjemo kada bilo koja operacija moze najprije zavrsiti
			najkrace = -1;
			for(job=0; job<jobs; job++)
			{	operation = (int) OperationsScheduled[job][0];
				if(operation == machines) continue;		// ovaj posao je zavrsio
				machine_index = (int) MachineIndex[job][operation];
				if(machine_index != machine) continue;	// ovaj posao ne ceka na taj stroj
				if(najkrace < 0)
					najkrace = MAX(time, JobReady[job][0]) + Durations[job][operation];
				else
				{	pocetak = MAX(time, JobReady[job][0]);
					if(najkrace > pocetak + Durations[job][operation])
						najkrace = pocetak + Durations[job][operation];
				}
			}
			// sada treba pronaci operaciju najveceg prioriteta koja ceka na taj stroj
			bValueSet = false;
			for(job=0; job<jobs; job++)
			{	operation = (int) OperationsScheduled[job][0];
				if(operation == machines) continue;		// ovaj posao je zavrsio
				machine_index = (int) MachineIndex[job][operation];
				if(machine_index != machine) continue;	// ovaj posao ne ceka na taj stroj
				// mozemo uzeti u obzir sam one koji su vec spremni:
					if(JobReady[job][0] > time) continue;	// jos nije zavrsila prethodna operacija
				// ili mozemo uzeti u obzir i one koji ce tek zavrsiti, ali ne kasnije od moguceg zavrsetka najkrace spremne operacije
				// (heuristika mora uracunati i idle trajanje)
				// rezultati koji se dobiju su negdje bolji, negdje losiji - treba ukljuciti obje strategije!
				pocetak = MAX(JobReady[job][0],time);
				if(pocetak > najkrace)	// nema smisla gledati ako pocinje iza najkraceg zavrsetka
					continue;
				if(!bValueSet)
				{	best = (this->*Fja)(set,job,operation,time,JobReady[job][0],(int)LastJob[machine][0]);
					best_job = job;
					bValueSet = true;
				}
				else
				{	value = (this->*Fja)(set,job,operation,time,JobReady[job][0],(int)LastJob[machine][0]);
					if(value > best)
					{	best = value;
						best_job = job;
					}
				}
			}
			operation = (int) OperationsScheduled[best_job][0];
			OperationsScheduled[best_job][0] += 1;	// novi broj rasporedjenih operacija posla
			if(setup)
			{	setuptime = Setup[(int)LastJob[machine][0]][best_job];
				LastJob[machine][0] = best_job;
			}
			else
				setuptime = 0;
			time = MAX(time, JobReady[best_job][0]);	// ako smo odlucili pricekati
			MachineReady[machine][0] = time + setuptime + Durations[best_job][operation];
			JobReady[best_job][0] = MachineReady[machine][0];	// kada ce sljedeca operacija biti raspoloziva
			if(operation < machines-1)	// ako posao ima jos operacija
			{	next_machine = (int) MachineIndex[best_job][ (int) OperationsScheduled[best_job][0]];
				if(OperationReady[next_machine][0] < 0)	// ako stroj trenutno nije trazen...
					OperationReady[next_machine][0] = JobReady[best_job][0];	// ... bit ce tada
				else	// provjeri dolazi li moja sljedeca operacija prije trenutno najblize za taj stroj
					if(OperationReady[next_machine][0] > JobReady[best_job][0])
						OperationReady[next_machine][0] = JobReady[best_job][0];
			}
			// kada cu ja (stroj) opet biti trazen?
			OperationReady[machine][0] = -1;
			for(j=0; j<jobs; j++)
			{	operation = (int) OperationsScheduled[j][0];
				if(operation == machines) continue;	// ovaj nema vise operacija
				next_machine = (int) MachineIndex[j][operation];
				if(next_machine != machine) continue;
				if(OperationReady[machine][0] < 0)
					OperationReady[machine][0] = JobReady[j][0];
				else
					if(OperationReady[machine][0] > JobReady[j][0])
						OperationReady[machine][0] = JobReady[j][0];
			}
			operation = (int) MachineScheduled[machine][0];
			MachineScheduled[machine][0] += 1;
			Schedule[set][machine * jobs + operation] = best_job;
		}	// kraj rasporedjivanja skupa
	}	// kraj od svih skupova
}