#pragma once
#include "check.h"
#include "matrice.h"
#include "readpar.h"
#include <string>

#define POS(x) x>0 ? x : 0

enum Environment
{	SINGLE,
	UNIFORM,
	UNRELATED,
	JOBSHOP
};

const int nCriteria = 4;

class Scheduler
{
public:
	int dynamic;
	int constraints;
	int setup;
	int weights;
	int max_length;
	int or_solutions;
	double sqs_faktor;
	Environment environment;
	std::string sORFile;
	Matrica R,T,N,SP,SD,Duration,Deadline,Weight,Ready,Machines,Speed,Constraints,Pkav;
	Matrica Precedence, Level, Setup, Schedule, Values, BestValues, Durations, MachineIndex;
	// pokazivac na funkcije koje odredjuju prioritet
	double (Scheduler::*heuristic) (int set, int job, int last_job, double time, int machine);

	void Test(char input[], char outfile[]);
	void MakeSets(char input[]);

	void MakeSetup(int set);
	double NodeLevel(int set,int node);
	void ReadConstraints(int set);
	double Evaluate(void);
	double EvaluateMultiple(void);
	double EvaluateJobShop(void);
	void UpdateBestValues(void);

	double ValueHL(int set, int job, int last_job, double time, int machine);
	double ValueLNS(int set, int job, int last_job, double time, int machine);
	double ValueWSPT(int set, int job, int last_job, double time, int machine);
	double ValueEDD(int set, int job, int last_job, double time, int machine);
	double ValueMON(int set, int job, int last_job, double time, int machine);
	double ValueRM(int set, int job, int last_job, double time, int machine);
	double ValueXD(int set, int job, int last_job, double time, int machine);
	double ValueMakespan(int set, int job, int last_job, double time, int machine);
	double ValueATCS(int set, int job, int last_job, double time, int machine);
	// job shop heuristike
	double JSValueWSPT(int set, int job, int operation, double time, double ready, int last);
	double JSValueWTWKR(int set, int job, int operation, double time, double ready, int last);
	double JSValueWSPT_TWKR(int set, int job, int operation, double time, double ready, int last);
	double JSValueCOVERT(int set, int job, int operation, double time, double ready, int last);
	double JSValueEDD(int set, int job, int operation, double time, double ready, int last);
	double JSValueSLACK_TWKR(int set, int job, int operation, double time, double ready, int last);
	double JSValueRM(int set, int job, int operation, double time, double ready, int last);

	void MakeScheduleNwt_H1(Matrica &Schedule, Matrica N, Matrica Duration, Matrica Deadline, Matrica SP, Matrica Weight, Matrica Ready, Matrica Constraints);
	void MakeSchedule(double (Scheduler::*Fja)(int set, int job, int last_job, double time, int machine) );
	void MakeScheduleMultiple(double (Scheduler::*Fja)(int set, int job, int last_job, double time, int machine) );
	void MakeScheduleJobShop(double (Scheduler::*Fja)(int set, int job, int operation, double time, double ready, int last) );
	void MakeScheduleMinMin(void);

private:
	double Savg;	// prosjecno trajanje postavljanja
	double Pav;		// prosjecno trajanje preostalih poslova
	double PTavg;	// prosjecno trajanje SVIH poslova (ne samo preostalih)
	double C1max;	// procjena Cmax
	double ATCS1,ATCS2;	// parametri za ATCS
	double M;		// = suma brzina svih strojeva
	double Mavg;	// prosjek brzina
};

// inline funkcije

// SPT
// max { weight / procesing time }
inline double Scheduler::JSValueWSPT(int set, int job, int operation, double time, double ready, int last)
{
	double idle = POS(ready - time);
	//double idle = 0;
	if(weights)
		return Weight[set][job] / (Durations[job][operation] + idle);
	else
		return 1 / (Durations[job][operation] + idle);
}

// SPT/TWKR
// max { weight * total work remaining / procesing time }
inline double Scheduler::JSValueWSPT_TWKR(int set, int job, int operation, double time, double ready, int last)
{
	double twkr = 0;
	double idle = POS(ready - time);
	//double idle = 0;
	int i, m = (int) Machines[set][0];
	for(i=operation; i<m; i++)
		twkr += Durations[job][i];
	twkr += idle;
	if(weights)
		return Weight[set][job] * twkr / (Durations[job][operation] + idle);
	else
		return twkr / (Durations[job][operation] + idle);
}

// WTWKR
// max { weight / total work remaining }
inline double Scheduler::JSValueWTWKR(int set, int job, int operation, double time, double ready, int last)
{
	double twkr = 0;
	double idle = POS(ready - time);
	//double idle = 0;
	int i, m = (int) Machines[set][0];
	for(i=operation; i<m; i++)
		twkr += Durations[job][i];
	twkr += idle;
	if(weights)
		return Weight[set][job] / twkr;
	else
		return 1 / twkr;
}

// COVERT pravilo
inline double Scheduler::JSValueCOVERT(int set, int job, int operation, double time, double ready, int last)
{
	int i, ops = (int) Machines[set][0];
	double lead = 0, result, k, h, slack;
	double idle = POS(ready - time);
	//double idle = 0;
	for(i=operation+1; i<ops; i++)
		lead += Durations[job][i];
	k = 4;	// procjena faktora za lead time
	h = 0.5;	// neovisni koeficijent heuristike
	slack = POS(Deadline[set][job] - k * lead - time);
	slack = POS(1. - slack / (h * lead));
	result = (1. / (Durations[job][operation] + idle)) * slack;
	if(weights)
		result *= Weight[set][job];
	return result;
}

// EDD
inline double Scheduler::JSValueEDD(int set, int job, int operation, double time, double ready, int last)
{
	return -Deadline[set][job];
}

// SLACK / TWKR
// max { weight * total work remaining / slack }
inline double Scheduler::JSValueSLACK_TWKR(int set, int job, int operation, double time, double ready, int last)
{
	double twkr = 0;
	double idle = POS(ready - time);
	//double idle = 0;
	int i, ops = (int) Machines[set][0];
	for(i=operation; i<ops; i++)
		twkr += Durations[job][i];
	twkr += idle;
	double slack = Deadline[set][job] - twkr;
	if(weights)
		return Weight[set][job] * twkr / slack;
	else
		return twkr / slack;
}

// R&M
inline double Scheduler::JSValueRM(int set, int job, int operation, double time, double ready, int last)
{
	int i, ops = (int) Machines[set][0], jobs = (int) N[set][0];
	int machine =  (int) MachineIndex[job][operation];
	double lead = 0, result, k, h, slack, Pavg = 0;
	double idle = POS(ready - time);
	//double idle = 0;
	for(i=operation+1; i<ops; i++)
		lead += Durations[job][i];
	k = 3;	// procjena faktora za lead time
	h = 10;	// faktor heuristike
	// za Twt ispada najbolje oko 10; ako se povecava, pogorsava se Twt, ali se poboljsava Fwt!
	slack = POS(Deadline[set][job] - k *lead - time);
	result = (1 / (Durations[job][operation] + idle)) * exp(-slack / (h * Pkav[machine][0]));
	if(weights)
		result *= Weight[set][job];
	return result;
}
