// Radi sljedece:
// 1. cita fajl s rjesenjima (HoF output) i uzima rjesenje iz zadane generacije
// 2. kopira ga u izlazni seed fajl
// 3. pokrece beagle (test.bat) za testiranje rjesenja
// 4. pokrece (fitness.exe) za ocjenu i usporedbu
//
// Zadnja promjena: 2011/02/17
//
#include <iostream>
#include <string>
#include <fstream>
#include <process.h>
#include <windows.h>
using namespace std;

string outFile = "seed.obm";
string testProg = "test.bat";
string fitness = "evaluacija.exe";
string fitnessOR = "fitness_OR.exe";


void RunProcess(string file)
{
    const char *cmd = file.data();
	STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );

    // Start the child process. 
    if( !CreateProcess( NULL,   // No module name (use command line). 
//		TEXT("test.bat"), // Command line. 
		(LPSTR) cmd, // Command line. 
        NULL,             // Process handle not inheritable. 
        NULL,             // Thread handle not inheritable. 
        FALSE,            // Set handle inheritance to FALSE. 
        0,                // No creation flags. 
        NULL,             // Use parent's environment block. 
        NULL,             // Use parent's starting directory. 
        &si,              // Pointer to STARTUPINFO structure.
        &pi )             // Pointer to PROCESS_INFORMATION structure.
    ) 
    {
        printf( "CreateProcess failed (%d).\n", GetLastError() );
        return;
    }

    // Wait until child process exits.
    WaitForSingleObject( pi.hProcess, INFINITE );

    // Close process and thread handles. 
    CloseHandle( pi.hProcess );
    CloseHandle( pi.hThread );

}

int GetLastGeneration(ifstream &in)
{
	string sLine;
	int nGen = 0;
	while(in)
	{	getline(in, sLine);
		if(sLine.compare(0, 14, "-- Generacija: "))
			sscanf(sLine.data() + 14, "%d", &nGen);
	}
	in.clear();
	return nGen;
}

string GetGenerationBest(ifstream &in, int nGen)
{
	string sLine = "";
	int nCurrGen;
	while(in)
	{	getline(in, sLine);
		if(sLine.compare(0, 14, "-- Generacija: "))
		{	sscanf(sLine.data() + 14, "%d", &nCurrGen);
			if(nCurrGen != nGen)
				continue;
			getline(in, sLine);
			getline(in, sLine);
			getline(in, sLine);
			getline(in, sLine);
			getline(in, sLine);
			getline(in, sLine);
			in.clear();
			return sLine;
		}
	}
	in.clear();
	return sLine;
}

int main()
{
	string inFile, buf, outFileLocal;
	ifstream in;
	ofstream out;
	int nGen, nLastGen;
	bool bOR = false;

	cout << " -- Program za automatsku evaluaciju jedinke genetskog programiranja (za OBeagle):\n";
	cout << " 1. cita datoteku s rjesenjima (HoF output) i uzima rjesenje iz zadane generacije\n";
	cout << " 2. kopira ga u izlazni seed fajl (" << outFile << ")\n";
	cout << " 3. pokrece beagle (" << testProg << ") za testiranje rjesenja\n";
	cout << " 4. pokrece program za ocjenu (" << fitness << ") i usporedbu\n\n";

	while(1)
	{
		in.clear();
		out.clear();
		do
		{	cout << "Ulazni fajl: ";
			getline(cin, inFile);
			in.open(inFile.c_str());
		}while(!in.is_open());

		do
		{	cout << "Izlazni fajl (" << outFile << "): ";
			getline(cin, buf);
			if(buf == "")
				outFileLocal = outFile;
			else
				outFileLocal = buf;
			out.open(outFileLocal.c_str(), ios_base::out);
		}while(!out.is_open());

		cout << "Test program (" << testProg << "): ";
		getline(cin, buf);
		if(buf == "")
			buf = testProg;
		else
			testProg = buf;

		cout << "Program za evaluaciju (" << fitness << "): ";
		getline(cin, buf);
		if(buf == "")
			buf = fitness;
		else
			fitness = buf;

		cout << "OR rjesenja (enter - ne, <> - da): ";
		getline(cin, buf);
		if(buf != "")
			bOR = true;

		nLastGen = GetLastGeneration(in);
		in.seekg(1, ios_base::beg);
		cout << "Broj generacija: " << nLastGen << endl;

		do
		{
			buf = "";
			cout << "Generacija (enter - zadnja, 0 - restart, -1 - kraj): ";
			getline(cin, buf);
			if(buf == "")
				nGen = nLastGen;
			else 
				sscanf(buf.data(), "%d", &nGen);
			if(nGen <= 0)
			{	in.close();
				out.close();
				break;
			}

			buf = GetGenerationBest(in, nGen);
			in.seekg(0);
			if(buf == "")
			{	cout << "Nema te generacije!" << endl;
				continue;
			}

			if(!out.is_open())
				out.open(outFileLocal.c_str(), ios_base::out);
			out << "?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" << endl;
			out << "<Beagle>" << endl << "<Seeds>" << endl << "<Individual>" << endl;
			out << buf << endl;
			out << "</Individual>" << endl << "</Seeds>" << endl << "</Beagle>" << endl;
			out.close();
			cout << "Zapisao u " << outFileLocal << endl;

			cout << "Pokrecem " << testProg << endl;
			//_execl(testProg.c_str(), testProg.c_str(), NULL);
			RunProcess(testProg);

			cout << "\nPokreni provjeru (" << fitness << "): ";
			getline(cin, buf);
			if(buf != "")
				RunProcess(buf);
			else
				RunProcess(fitness);

			if(bOR)
			{	cout << "\nPokreni provjeru s OR rjesenjima (" << fitnessOR << "): ";
				getline(cin, buf);
				if(buf != "")
					RunProcess(buf);
				else
					RunProcess(fitnessOR);
			}

		}while(nGen > 0);

		if(nGen < 0)
			break;

	} // while(1)


	return 0;
}