// Radi sljedece:
// 1. cita fajl s rjesenjima i uzima rjesenja iz svih generacija
// 2. pokrece beagle (test.bat) i (fitness.exe) za sva rjesenja
// 3. rezultate pise u odredisni fajl
//
// Zadnja promjena: 2008/02/10
//
#include <iostream>
#include <string>
#include <fstream>
#include <process.h>
#include <windows.h>
using namespace std;

string seedFile = "seed.obm";
string testProg = "test.bat";
string fitness = "fitness.exe";
string fitnessOR = "fitness_OR.exe";


void RunProcess(string file)
{
    const char *cmd = file.data();
	STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );

    // Start the child process. 
    if( !CreateProcess( NULL,   // No module name (use command line). 
//		TEXT("test.bat"), // Command line. 
		(LPSTR) cmd, // Command line. 
        NULL,             // Process handle not inheritable. 
        NULL,             // Thread handle not inheritable. 
        FALSE,            // Set handle inheritance to FALSE. 
        0,                // No creation flags. 
        NULL,             // Use parent's environment block. 
        NULL,             // Use parent's starting directory. 
        &si,              // Pointer to STARTUPINFO structure.
        &pi )             // Pointer to PROCESS_INFORMATION structure.
    ) 
    {
        printf( "CreateProcess failed (%d).\n", GetLastError() );
        return;
    }

    // Wait until child process exits.
    WaitForSingleObject( pi.hProcess, INFINITE );

    // Close process and thread handles. 
    CloseHandle( pi.hProcess );
    CloseHandle( pi.hThread );

}

int GetLastGeneration(ifstream &in)
{
	string sLine;
	int nGen = 0;
	while(in)
	{	getline(in, sLine);
		if(sLine.compare(0, 14, "-- Generacija: "))
			sscanf(sLine.data() + 14, "%d", &nGen);
	}
	in.clear();
	return nGen;
}

int GetNextGeneration(ifstream &in)
{
	string sLine;
	int nGen = 0;
	do
		getline(in, sLine);
	while(sLine.find("Generacija") == -1 || !in);
	if(in)
		sscanf(sLine.data() + 14, "%d", &nGen);
	in.clear();
	return nGen;
}

string GetNextSolution(ifstream &in)
{
	string sLine;
	do
		getline(in, sLine);
	while(sLine.find("Genotype") == -1 || !in);
	if(in)
		return sLine;
	return "";
}

double GetNextFitness(ifstream &in)
{
	double dFitness = 0;
	string sLine;
	do
		getline(in, sLine);
	while(sLine.find("Fitness") == -1 || !in);
	if(in)
	{	int pos = (int) sLine.find_first_of(">");
		sscanf(sLine.data() + pos + 1, "%lf", &dFitness);
	}
	return dFitness;
}

int GetSolutionSize(string solution)
{
	int size = 0;
	int pos = (int) solution.find("size=") + 6;
	sscanf(solution.data() + pos, "%d", &size);
	return size;
}

string GetGenerationBest(ifstream &in, int nGen)
{
	string sLine = "";
	int nCurrGen;
	while(in)
	{	getline(in, sLine);
		if(sLine.compare(0, 14, "-- Generacija: "))
		{	sscanf(sLine.data() + 14, "%d", &nCurrGen);
			if(nCurrGen != nGen)
				continue;
			getline(in, sLine);
			getline(in, sLine);
			getline(in, sLine);
			getline(in, sLine);
			getline(in, sLine);
			getline(in, sLine);
			in.clear();
			return sLine;
		}
	}
	in.clear();
	return sLine;
}

int main()
{
	string inFile, buf, outFile;
	ifstream in;
	ofstream out, seed;
	int nGen, nLastGen, iSize;
	bool bOR = false;
	double dFitness;

	while(1)
	{
		in.clear();
		out.clear();
		seed.clear();
		do
		{	cout << "Ulazni fajl: ";
			getline(cin, inFile);
			in.open(inFile.c_str());
		}while(!in.is_open());

		do
		{	cout << "Izlazni fajl: ";
			getline(cin, outFile);
			out.open(outFile.c_str(), ios_base::out);
		}while(!out.is_open());
		fitness = fitness + " " + outFile;
		out << "Gen\tFit\tSize\tTwt\tNwt\tFwt\tCmax\tTwt\tNwt\tFwt\tCmax";

		cout << "Test program (" << testProg << "): ";
		getline(cin, buf);
		if(buf == "")
			buf = testProg;
		else
			testProg = buf;

		nLastGen = GetLastGeneration(in);
		in.seekg(1, ios_base::beg);
		cout << "Broj generacija: " << nLastGen << endl;

		do
		{
			// read next solution
			nGen = GetNextGeneration(in);
			if(nGen == 0)
				break;
			dFitness = GetNextFitness(in);
			buf = GetNextSolution(in);
			iSize = GetSolutionSize(buf);
			cout << "Procitao generaciju " << nGen << endl;

			if(!out.is_open())
				out.open(outFile.c_str(), ios_base::app);
			out << endl << nGen << "\t" << dFitness << "\t" << iSize;
			out.close();

			// write solution in seed file
			if(!seed.is_open())
				seed.open(seedFile.c_str(), ios_base::out);
			seed << "?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" << endl;
			seed << "<Beagle>" << endl << "<Seeds>" << endl << "<Individual>" << endl;
			seed << buf << endl;
			seed << "</Individual>" << endl << "</Seeds>" << endl << "</Beagle>" << endl;
			seed.close();
			cout << "Zapisao u " << seedFile << endl;

			cout << "Pokrecem " << testProg << endl;
			RunProcess(testProg);

			cout << "\nPokrecem provjeru (" << fitness << ")" << endl;
			RunProcess(fitness);

		}while(nGen < nLastGen);

		in.close();

	} // while(1)


	return 0;
}