// Radi sljedece:
// 1. cita fajlove s rjesenjima tipa 00.txt, 01.txt itd dok ih ima
// 2. uzima rjesenje iz zadane ili zadnje generacije 
// 2. kopira ga u odredisni seed fajl
// 3. pokrece beagle (test.bat) za testiranje rjesenja
// 4. pokrece (fitness.exe) za ocjenu i usporedbu
// 5. sprema rjesenja u izlazni fajl
//
// Zadnja promjena: 2008/02/15
//
#include <iostream>
#include <string>
#include <fstream>
#include <process.h>
#include <windows.h>
using namespace std;

string seedFile = "seed.obm";
string testProg = "test.bat";
string fitness = "fitness.exe";
string fitnessOR = "fitness_OR.exe";


void RunProcess(string file)
{
    const char *cmd = file.data();
	STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory( &si, sizeof(si) );
    si.cb = sizeof(si);
    ZeroMemory( &pi, sizeof(pi) );

    // Start the child process. 
    if( !CreateProcess( NULL,   // No module name (use command line). 
//		TEXT("test.bat"), // Command line. 
		(LPSTR) cmd, // Command line. 
        NULL,             // Process handle not inheritable. 
        NULL,             // Thread handle not inheritable. 
        FALSE,            // Set handle inheritance to FALSE. 
        0,                // No creation flags. 
        NULL,             // Use parent's environment block. 
        NULL,             // Use parent's starting directory. 
        &si,              // Pointer to STARTUPINFO structure.
        &pi )             // Pointer to PROCESS_INFORMATION structure.
    ) 
    {
        printf( "CreateProcess failed (%d).\n", GetLastError() );
        return;
    }

    // Wait until child process exits.
    WaitForSingleObject( pi.hProcess, INFINITE );

    // Close process and thread handles. 
    CloseHandle( pi.hProcess );
    CloseHandle( pi.hThread );

}

int GetLastGeneration(ifstream &in)
{
	string sLine;
	int nGen = 0;
	while(in)
	{	getline(in, sLine);
		if(sLine.compare(0, 14, "-- Generacija: "))
			sscanf(sLine.data() + 14, "%d", &nGen);
	}
	in.clear();
	return nGen;
}

string GetGenerationBest(ifstream &in, int nGen)
{
	string sLine = "";
	int nCurrGen;
	while(in)
	{	getline(in, sLine);
		if(sLine.compare(0, 14, "-- Generacija: "))
		{	sscanf(sLine.data() + 14, "%d", &nCurrGen);
			if(nCurrGen != nGen)
				continue;
			getline(in, sLine);
			getline(in, sLine);
			getline(in, sLine);
			getline(in, sLine);
			getline(in, sLine);
			getline(in, sLine);
			in.clear();
			return sLine;
		}
	}
	in.clear();
	return sLine;
}

int main()
{
	string outFile, buf;
	char inFile[1000];
	ifstream in;
	ofstream out, seed;
	int nGen, nLastGen, nFirstDigit = 0, nSecondDigit = 0;

	while(1)
	{
		in.clear();
		out.clear();
		seed.clear();

		do
		{	cout << "Izlazni fajl (" << outFile << "): ";
			getline(cin, outFile);
			out.open(outFile.c_str(), ios_base::out);
		}while(!out.is_open());
		fitness = fitness + " " + outFile;
		out << "\tTwt\tNwt\tFwt\tCmax\tTwt\tNwt\tFwt\tCmax\n";
		out.close();

		cout << "Test program (" << testProg << "): ";
		getline(cin, buf);
		if(buf == "")
			buf = testProg;
		else
			testProg = buf;

		cout << "Generacija (enter - zadnja): ";
		getline(cin, buf);
		if(buf == "")
			nGen = -1;
		else 
			sscanf(buf.data(), "%d", &nGen);

		nFirstDigit = nSecondDigit = 0;
		while(1)
		{	sprintf(inFile, "%d%d.txt", nFirstDigit, nSecondDigit);
			nSecondDigit++;
			if(nSecondDigit == 10)
			{	nSecondDigit = 0;
				nFirstDigit++;
			}
			if(nFirstDigit == 10)
				break;

			in.clear();
			in.open(inFile);
			if(!in.is_open())
				continue;

			cout << "Otvorio " << inFile << endl;

			nLastGen = GetLastGeneration(in);
			in.seekg(1, ios_base::beg);
			cout << "Broj generacija: " << nLastGen << endl;

			if(nGen > 0)
				buf = GetGenerationBest(in, nGen);
			else
				buf = GetGenerationBest(in, nLastGen);
			in.close();

			if(buf == "")
			{	cout << "Nema te generacije! (" << nGen << ")" << endl;
				continue;
			}

			// write solution in seed file
			if(!seed.is_open())
				seed.open(seedFile.c_str(), ios_base::out);
			seed << "?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" << endl;
			seed << "<Beagle>" << endl << "<Seeds>" << endl << "<Individual>" << endl;
			seed << buf << endl;
			seed << "</Individual>" << endl << "</Seeds>" << endl << "</Beagle>" << endl;
			seed.close();
			cout << "Zapisao generaciju " << nGen << " u " << seedFile << endl;

			cout << "Pokrecem " << testProg << endl;
			RunProcess(testProg);

			cout << "\nPokrecem provjeru (" << fitness << ")" << endl;
			RunProcess(fitness);
			out.open(outFile.c_str(), ios_base::app);
			out << endl;
			out.close();
		}

	} // while(1)


	return 0;
}