#include "beagle/Beagle.hpp"
#include "AfterGenOp.h"
#include "WeightedPrimitiveSet.h"
#include <iomanip>

using namespace Beagle;

#ifdef ADAPTATION
extern GP::WeightedPrimitiveSet::Handle Set;
#else
extern GP::PrimitiveSet::Handle Set;
#endif

const int GEN = 0;
const int AVG = 1;
const int STD = 2;
const int MIN = 3;
const int MAX = 4;
const int SUM2 = 5;
#define MIN(x,y) x<y ? x : y
#define MAX(x,y) x>y ? x : y

/*!
 *  \brief Construct a termination check operator based on a maximum consecutive generations without improvement of the best-so-far indivudual.
 *  \param inName Name of the operator.
 */
AfterGenOp::AfterGenOp(std::string inName) :
  TerminationOp(inName)
{ }

/*!
 *  \brief Initialize the maximum generation without improvement termination operator.
 *  \param ioSystem System to use to initialize the operator.
 */
void AfterGenOp::initialize(System& ioSystem)
{
	m_nHOFFrequency = new UInt(2);	// svakih 2 generacija pisemo najbolje
	m_nLastEditedNodes = m_nLastTotalNodes = 0;
	m_bGreska = false;
}

void AfterGenOp::Adaptation(Context& ioContext)
{
	// isprobavanje - dohvacanje statistickih vrijednosti
	double dAvgFt, dMinFt, dMaxFt;
	dAvgFt = (*ioContext.getVivarium().getStats())[0].mAvg;
	dMinFt = (*(ioContext.getVivarium().getStats()))[0].mMin;
	dMaxFt = (*(ioContext.getVivarium().getStats()))[0].mMax;
	//m_dCurrent = fabs(dMaxFt - dAvgFt) / fabs(dMaxFt - dMinFt);
	m_dCurrent = fabs(dMaxFt - dAvgFt);
	if(m_dPrevious < 0)	// ako jos nismo racunali - zbog opcenitosti postupka
		m_dPrevious = m_dCurrent;
	if(m_dCurrent > m_dPrevious)	// korekcija
		m_dPrevious = m_dCurrent;
	m_dW = pow(m_dCurrent / m_dPrevious, 2);
	m_dPrevious = m_dCurrent;	//nova sljedeca vrijednost

	//int dummy;
	//char *strW = _fcvt(m_dW, 5, &dummy, &dummy);
	char strW[16];
	sprintf(strW,"W = %8g",m_dW);
	Beagle_LogInfoM(
      ioContext.getSystem().getLogger(),
      "termination", "Beagle::AfterGenOp",
      strW
    );

	double dA = 0.7, dB = 0.05;	// najveca i osnovna vjerojatnost mutacije
	double dPM = (dA - dB) * (1 - m_dW) + dB;
	// isprobavanje - mijenjanje vrijednosti parametra
	Double::Handle hMut;
	hMut = new Double(dPM);
	ioContext.getSystem().getRegister().modifyEntry("gp.mutstd.indpb", hMut);

}

void AfterGenOp::Statistic(Context& ioContext)
{
#ifdef ADAPTATION
	unsigned int nEdited,nTotal,nPercentage, nTotalPercentage, nUseful;
	unsigned int i,j;
	double dEditedRate, dValSum, dMaxCumVal;
	double dSubsetValues[TOTAL_NODES];

	nEdited = EvalOp->edited - m_nLastEditedNodes;	// broj editiranih cvorova u protekloj generaciji
	nTotal = EvalOp->total - m_nLastTotalNodes;		// broj svih cvorova u protekloj gen.
	nUseful = nTotal - nEdited;
	dEditedRate = (double)nEdited / nTotal;
	nPercentage = (unsigned int) (dEditedRate*100);
	m_nLastTotalNodes = EvalOp->total;
	m_nLastEditedNodes = EvalOp->edited;
	nTotalPercentage = (unsigned int) (100 * (double)EvalOp->edited / EvalOp->total);
	dMaxCumVal = 0;

	Beagle_LogInfoM(
      ioContext.getSystem().getLogger(),"termination", "Beagle::AfterGenOp",
      std::string("Postotak editiranih cvorova = ") + uint2str(nPercentage) +
	  std::string("%, kumulativno = ") + uint2str(nTotalPercentage) + std::string("%")
    );

	// odradi statistiku terminala
	unsigned int nGen = ioContext.getGeneration();
	if(nGen == 1)	// zapocni pratiti terminale u najboljim jedinkama
	{	//EvalOp->StartTerminalStatistic(castObjectT<GP::Context&>(ioContext));
		EvalOp->StartTerminalStatistic(castObjectT<GP::Context&>(ioContext), 1);	// cijela populacija
		m_dCumSum = 0;
		for(i=0; i<TOTAL_NODES; i++)
			m_dCumSubsetValue[i] = 0;
	}
	
	// skupljanje vrijednosti
	// prodji sve terminale i funkcije za cijelu populaciju
	for(i=0; i<TOTAL_NODES; i++)
	{	if(Nodes[i].active)
			m_dNodePercentage[i] = 1.0 * Nodes[i].frequency / nUseful;
	}
	// ovo samo od kada zelimo
	if(nGen>1)
	{	EvalOp->GetTerminalStatistic(dSubsetValues);
		dValSum = 0; dMaxCumVal = 0;
		for(i=0; i<TOTAL_NODES; i++)	// prodji sve terminale i funkcije, samo za podskup najboljih
		{	if(Nodes[i].active)
			{	//m_dCumSubsetValue[i] = (m_dCumSubsetValue[i]*(nGen-2) + dSubsetPercentage[i]) / (nGen-1);
				m_dCumSubsetValue[i] += dSubsetValues[i];
				if(i<FUNC_START)	// samo za terminale! zasada...
				{	if(m_dCumSubsetValue[i] > dMaxCumVal)
						dMaxCumVal = m_dCumSubsetValue[i];
					m_dCumSum += dSubsetValues[i];
					dValSum += dSubsetValues[i];
				}
			}
		}
	}

	// adaptivni dio... pomocu kumulativnih vrijednosti
/*	for(i=OFFSET; i<OFFSET+TERMINALS; i++)
	{	if(!Nodes[i].active) continue;
		if(m_dCumSubsetValue[i]/m_dCumSum < m_dNodePercentage[i])
			Set->ChangeWeight(i-OFFSET, 100);
//		else if(m_dCumSubsetValue[i]/m_dCumSum > m_dNodePercentage[i])
//			Set->ChangeWeight(i-OFFSET, 0.1);
		else
			Set->ChangeWeight(i-OFFSET, 1);
	}
*/
	// adaptivni dio... po uzoru na Miyashitu
/*	for(i=OFFSET; i<OFFSET+TERMINALS; i++)
	{	if(Nodes[i].active)
			Set->ChangeWeight(i-OFFSET, 1 + 30 * m_dCumSubsetValue[i]/m_dCumSum);
	}
	EvalOp->StartTerminalStatistic(castObjectT<GP::Context&>(ioContext));
*/

//
// adaptivni dio - prostor za eksperimentiranje
//
	// mijenjanje velicine bez resetiranja subseta svake generacije
	if(nGen == 11)
		EvalOp->StartTerminalStatistic(castObjectT<GP::Context&>(ioContext));

	// mijenjanje velicine uz resetiranje subseta
/*	if(nGen<=10)
		EvalOp->StartTerminalStatistic(castObjectT<GP::Context&>(ioContext), 1);
	else
		EvalOp->StartTerminalStatistic(castObjectT<GP::Context&>(ioContext));
*/
	// adaptacija + ogranicavanje (cuvanje je iza)
	if(nGen>1 && m_dCumSum != 0 && dValSum != 0)
	{	for(i=OFFSET; i<OFFSET+TERMINALS; i++)
		{	if(!Nodes[i].active) continue;
			if(m_dNodePercentage[i] > 0.1)
				Set->ChangeWeight(i-OFFSET, 1);
			else
			{	double dValue = 1 + 30 * m_dCumSubsetValue[i]/m_dCumSum;
				//double dValue = 1 + 30 * dSubsetValues[i]/dValSum;	// nekumulativno
				Set->ChangeWeight(i-OFFSET, dValue);
			}
		}
	}
	// drugi oblik adaptacije (uz ogranicavanje)
/*	if(nGen>1 && m_dCumSum != 0 && dValSum != 0)
		for(i=OFFSET; i<OFFSET+TERMINALS; i++)
		{	if(!Nodes[i].active) continue;
			if(m_dNodePercentage[i] > 0.1)
				Set->ChangeWeight(i-OFFSET, 1);
			if(m_dCumSubsetValue[i]/m_dCumSum < m_dNodePercentage[i])
				Set->ChangeWeight(i-OFFSET, 10);
			else if(m_dCumSubsetValue[i]/m_dCumSum > m_dNodePercentage[i])
				Set->ChangeWeight(i-OFFSET, 0.1);
			else
				Set->ChangeWeight(i-OFFSET, 1);
		}
*/
	// cuvanje terminala do neke (?) generacije
	double dMaxValue = 1 + 30 * dMaxCumVal/m_dCumSum;
	//double dMaxValue = 10;
	if(nGen<=10)
		for(i=OFFSET; i<OFFSET+TERMINALS; i++)
		{	if(Nodes[i].active && m_dNodePercentage[i]<0.01)
				Set->ChangeWeight(i-OFFSET, dMaxValue/2);
		}
	else
		for(i=OFFSET; i<OFFSET+TERMINALS; i++)
		{	if(Nodes[i].active && m_dNodePercentage[i]<0.01)
				Set->ChangeWeight(i-OFFSET, dMaxValue);
		}

	// resetiranje?
	if( (nGen) == 11)	// nakon promjene velicine subseta
	{	for(i=OFFSET; i<TOTAL_NODES; i++)
			m_dCumSubsetValue[i] = 0;
		m_dCumSum = 0;
	}
	// primjer ciljane adaptacije
/*	for(i=OFFSET; i<OFFSET+4; i++)
		if(dSubsetPercentage[i] < m_dNodePercentage[i])
			Set->ChangeWeight(i-OFFSET, 100);
		else if(dSubsetPercentage[i] > m_dNodePercentage[i])
			Set->ChangeWeight(i-OFFSET, 0.1);
		else
			Set->ChangeWeight(i-OFFSET, 1);
*/

//
// ispis
//
	String::Handle SFile = castHandleT<String>(ioContext.getSystem().getRegister().getEntry("lg.file.name"));
	std::string strFile = SFile->getWrappedValue();
	strFile += "_stat.txt";
	std::ofstream dat;
	// ako je prvi put, napisemo zaglavlje
	if(nGen == 1)
	{	dat.open(strFile.c_str());
		for(j=0; j<2; j++)	// 2 puta: za cijelu populaciju i za podskup najboljih
		{	for(i=0; i<TOTAL_NODES; i++)
				if(Nodes[i].active)
					dat << setw(3) << Nodes[i].name;
			dat << "|\t";
		}
		dat << "uk" << endl;
	}
	else
        dat.open(strFile.c_str(), std::ios_base::app);

	if(nGen != 1)
	{	uint nVal, nCumVal, nSuma = 0;
		for(i=0; i<TOTAL_NODES; i++)	// ispisi samo za subset
		{	if(!Nodes[i].active) continue;
			nCumVal = (uint) (100 * m_dCumSubsetValue[i] / m_dCumSum);
			nVal = (uint) (100 * dSubsetValues[i] / dValSum);
			dat << setw(3) << nVal << '(' << setw(2) << nCumVal << ')';
			nSuma += nCumVal;
		}
	}
	dat << "|\t";	// samo granicnik
	// ispisi sve terminale i funkcije za cijelu populaciju
	for(i=0; i<TOTAL_NODES; i++)
	{	if(Nodes[i].active)
		{	nPercentage = (uint) (m_dNodePercentage[i] * 100 + 0.5);
			dat << setw(3) << nPercentage;
		}
	}

	dat << nUseful;
	dat << endl;
	dat << std::flush;
	dat.close();
	EvalOp->Evaluator.ResetNodeFreq();
#endif
}

void AfterGenOp::SuccessRate(Context& ioContext)
{
	// dohvati najbolji fitness (iz HOF-a)
	Fitness::Handle hFit = ( (ioContext.getVivarium().getHallOfFame())[0].mIndividual)->getFitness();
	FitnessSimple FitSimp = castObjectT<Beagle::FitnessSimple&>(*hFit);
	double dFitness = FitSimp.getValue();

	// dohvati max i trenutni broj generacija
	UInt::Handle hMaxGen = castHandleT<UInt>(ioContext.getSystem().getRegister().getEntry("ec.term.maxgen"));
	uint nMaxGen = hMaxGen->getWrappedValue();
	uint nGen = ioContext.getGeneration();
	
	if(nGen == 1)
		Success.Reset(nMaxGen + 1,7);
	Success[nGen][GEN] = nGen;
	Success[nGen][AVG] = Success[nGen][MIN] = Success[nGen][MAX] = dFitness;
	Success[nGen][SUM2] = pow(Success[nGen][AVG], 2);
	if(fabs(dFitness) < 0.0001)	// ako je nasao rjesenje (samo za ogledni problem!)
	{	Success[0][1] = 1;	// oznaka da je nasao rjesenje
		Success[nGen][6] = 1;	// oznaka kada je nasao rjesenje
		for(int i=nGen+1; i<Success.GetRow(); i++)
			Success[i][0] = i;
	}
	if(m_bGreska)	// ?...
		Success[0][5] = 1;

	if(nGen == nMaxGen || Success[0][1] > 0)	// treba zapisati u fajl
	{	Matrica Previous;
		if(Previous.Load("success.txt"))	// vec ima nesto
		{	double dRuns = Previous[0][0], dValue;
			double dSum = 0, dSum2 = 0, dStdDev;
			for(int i=1; i<Previous.GetRow(); i++)
			{	dValue = Previous[i][AVG];
				Previous[i][MIN] = MIN(Previous[i][MIN], Success[i][MIN]);
				Previous[i][MAX] = MAX(Previous[i][MAX], Success[i][MAX]);
				dSum = Success[i][AVG] + dValue * dRuns;
				dSum2 = Success[i][SUM2] + Previous[i][SUM2];
				Previous[i][SUM2] = dSum2;	// nova suma kvadrata
				Previous[i][AVG] = dSum / (dRuns + 1);	// novi prosjek
				dStdDev = (dSum2 - pow(dSum,2)/(dRuns + 1)) / dRuns;
				dStdDev = sqrt(dStdDev);
				Previous[i][STD] = dStdDev;
				Previous[i][6] += Success[i][6];
			}
			Previous[0][0] = dRuns + 1;
			Previous[0][1] += Success[0][1];
			Previous[0][5] += Success[0][5];
			Previous.Save("success.txt");
		}
		else	// nije jos nista zapisano
		{	Success[0][0] = 1;
			Success.Save("success.txt");
		}
	}
}

/*!
 *  \brief 
 *  \param inDeme Actual deme of the evolution.
 *  \param ioContext Actual evolution context.
 *  \return
 */
void AfterGenOp::operate(Deme& inDeme, Context& ioContext)
{

	Beagle_LogInfoM(
      ioContext.getSystem().getLogger(),
      "termination", "Beagle::AfterGenOp",
      std::string("After generation operator")
    );

	//SuccessRate(ioContext);
#ifdef ADAPTATION
	Statistic(ioContext);
	Adaptation(ioContext);
#endif

	//Beagle::Individual::Handle hInd = (ioContext.getVivarium().getHallOfFame())[0].mIndividual;
	//EvalOp->evaluate(castObjectT<GP::Individual&>(*((ioContext.getVivarium().getHallOfFame())[0].mIndividual)), castObjectT<GP::Context&>(ioContext));
	//EvalOp->evaluate(castObjectT<GP::Individual&>(*hInd), castObjectT<GP::Context&>(ioContext));

	// ispisi HallOfFame u datoteku ako je trenutna generacija djeljiva sa HOFFrequency
	if((m_nHOFFrequency->getWrappedValue() != 0) && (ioContext.getGeneration() % m_nHOFFrequency->getWrappedValue() == 0))
	{	//std::string strFile = "hof_g" + uint2str(ioContext.getGeneration()) + ".txt";
		String::Handle SFile = castHandleT<String>(ioContext.getSystem().getRegister().getEntry("lg.file.name"));
		std::string strFile = SFile->getWrappedValue();
		strFile += "_hof.txt";
		std::ostream* lOS = new std::ofstream(strFile.c_str(), std::ios_base::app);
		XMLStreamer lStreamer(*lOS);
		*lOS << "-- Generacija: " << uint2str(ioContext.getGeneration()) << " --" << std::endl;
		//EvalOp->Evaluator.write();
		//*lOS << std::endl << "-- infix: " << EvalOp->Evaluator.output << " --" << std::endl;
		*lOS << "-- Ukupni broj cvorova: " << EvalOp->total << ", editirano: " << EvalOp->edited << endl;
		ioContext.getVivarium().getHallOfFame().write(lStreamer);
		//ioContext.getDeme().getHallOfFame().write(lStreamer);
		*lOS << endl;
		*lOS << std::flush;
		delete lOS;
		lOS = NULL;
	}
}