/*!
 *  \file   beagle/AfterGenOp.hpp
 *  \brief  Definition of the class AfterGenOp.
 *  \author 
 *  $Revision: 1.0 $
 *  $Date: 2005/01/20 $
 */

#ifndef Beagle_AfterGenOp_hpp
#define Beagle_AfterGenOp_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/WrapperT.hpp"
#include "beagle/Operator.hpp"
#include "beagle/TerminationOp.hpp"
#include "beagle/IntegerVector.hpp"
#include "beagle/UInt.hpp"
#include "beagle/Context.hpp"
#include "beagle/System.hpp"

#include "fitnes.hpp"

extern PrviEvalOp::Handle EvalOp;
extern node Nodes[TOTAL_NODES];

namespace Beagle {

/*!
 *  \class AfterGenOp beagle/AfterGenOp.hpp "beagle/AfterGenOp.hpp"
 *  \brief 
 *  \ingroup ECF
 *  \ingroup Op
 */
class AfterGenOp : public TerminationOp {

public:

  //! AfterGenOp allocator type
  typedef AllocatorT<AfterGenOp,TerminationOp::Alloc>
          Alloc;
  //! AfterGenOp handle type.
  typedef PointerT<AfterGenOp,TerminationOp::Handle>
          Handle;
  //! AfterGenOp bag type.
  typedef ContainerT<AfterGenOp,TerminationOp::Bag>
          Bag;

  explicit AfterGenOp(std::string inName="AfterGenOp");
  virtual ~AfterGenOp() { }

  virtual void initialize(System& ioSystem);
  virtual void operate(Deme& ioDeme, Context& ioContext);
  virtual bool terminate(const Deme& inDeme, Context& ioContext)
  { return false; }

protected:
  UInt::Handle m_nHOFFrequency;		//!< koliko cesto pisemo Hall of Fame
  unsigned int m_nLastTotalNodes, m_nLastEditedNodes;
  double m_dNodePercentage[TOTAL_NODES];	// postotak koristenja terminala
  double m_dCumSubsetValue[TOTAL_NODES];	// kumulativne vrijedosti terminala
  double m_dCumSum;	// zbroj kumulativnih vrijednosti
  bool m_bGreska;
    // varijable za prilagodbu operatora
	double m_dPrevious, m_dCurrent, m_dW;
/*
- prvi element matrice je broj vec zapisanih run-ova
- ostali su prosjek najboljih clanova svih ranova u odredjenoj generaciji
*/
  Matrica Success;	// evidencija prosjeka najboljih jedinki kroz generacije vise ranova

  void Statistic(Context& ioContext);	// racunanje i ispis statistike
  void SuccessRate(Context& ioContext);	// ispis kumulativne uspjesnosti (kroz vise run-ova)
  void Adaptation(Context& ioContext);	// prilagodba vjerojatnosti operatora

};

}

#endif