/*!
 *  \file   TermMaxNoImpGenOp.hpp
 *  \brief  Definition of the class TermMaxNoImpGenOp.
 *  \author 
 *  $Revision: 1.0 $
 *  $Date: 2005/01/20 $
 */

#ifndef Beagle_TermMaxNoImpGenOp_hpp
#define Beagle_TermMaxNoImpGenOp_hpp

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/AllocatorT.hpp"
#include "beagle/PointerT.hpp"
#include "beagle/ContainerT.hpp"
#include "beagle/WrapperT.hpp"
#include "beagle/Operator.hpp"
#include "beagle/TerminationOp.hpp"
#include "beagle/IntegerVector.hpp"
#include "beagle/UInt.hpp"
#include "beagle/Context.hpp"
#include "beagle/System.hpp"

namespace Beagle {

/*!
 *  \class TermMaxNoImpGenOp beagle/TermMaxNoImpGenOp.hpp "beagle/TermMaxNoImpGenOp.hpp"
 *  \brief Maximum consecutive generation without improvement of the best-so-far indivudual termination criterion operator.
 *  \ingroup ECF
 *  \ingroup Op
 */
class TermMaxNoImpGenOp : public TerminationOp {

public:

  //! TermMaxNoImpGenOp allocator type
  typedef AllocatorT<TermMaxNoImpGenOp,TerminationOp::Alloc>
          Alloc;
  //! TermMaxNoImpGenOp handle type.
  typedef PointerT<TermMaxNoImpGenOp,TerminationOp::Handle>
          Handle;
  //! TermMaxNoImpGenOp bag type.
  typedef ContainerT<TermMaxNoImpGenOp,TerminationOp::Bag>
          Bag;

  explicit TermMaxNoImpGenOp(std::string inName="TermMaxNoImpGenOp");
  virtual ~TermMaxNoImpGenOp() { }

  virtual void initialize(System& ioSystem);
  virtual bool terminate(const Deme& inDeme, Context& ioContext);

protected:
  UInt::Handle mMaxGenerationNoImp;		//!< Maximum number of consecutive generations in which no new best-so-far solution is found.
  double dFitnessChange, dFitnessNow;
  //std::vector<double> dFitnessPast;
  double *dFitnessPast;

};

}

#endif