/*!
 *  \file   WeightedPrimitiveSet.h
 *  \brief  Definition of the type GP::WeightedPrimitiveSet.
 *  \author 
 *  $Revision: 1.0 $
 *  $Date: 2004/07/01 23:59:00 $
 */

#ifndef Beagle_WeightedPrimitiveSet_h
#define Beagle_WeightedPrimitiveSet_h

#include <string>

#include "beagle/config.hpp"
#include "beagle/macros.hpp"
#include "beagle/Object.hpp"
#include "beagle/Allocator.hpp"
#include "beagle/Pointer.hpp"
#include "beagle/Container.hpp"
#include "beagle/System.hpp"
#include "beagle/GP/Primitive.hpp"


namespace Beagle {
namespace GP {


// Forward declaration
class System;


/*!
 *  \class WeightedPrimitiveSet WeightedPrimitiveSet.h "WeightedPrimitiveSet.h"
 *  \brief The GP primitive set class.
 *  \ingroup GPF
 *  \ingroup Primit
 *  \ingroup GPSys
 */
//class WeightedPrimitiveSet : public Primitive::Bag {
class WeightedPrimitiveSet : public GP::PrimitiveSet {

public:

  //! GP::WeightedPrimitiveSet allocator type.
  typedef AllocatorT<WeightedPrimitiveSet,GP::PrimitiveSet::Alloc>
          Alloc;
  //! GP::WeightedPrimitiveSet handle type.
  typedef PointerT<WeightedPrimitiveSet,GP::PrimitiveSet::Handle>
          Handle;
  //! GP::WeightedPrimitiveSet mixed bag type.
  typedef ContainerT<WeightedPrimitiveSet,GP::PrimitiveSet::Bag>
          Bag;

  explicit WeightedPrimitiveSet(std::string inRootType="");
  virtual ~WeightedPrimitiveSet() { }

  /*!
   *  \brief Set the associated tree's root type.
   *  \param inRootType Root type.
   */
  inline void setRootType(std::string inRootType)
  {
    mRootType = inRootType;
  }

  virtual std::string       getRootType() const;
          Primitive::Handle getPrimitiveByName(std::string inName) const;
  virtual void              initialize(GP::System& ioSystem);
  virtual void              insert(Primitive::Handle inPrimitive, double inWeight=1.0);
  virtual Primitive::Handle selectPrimitive(GP::System& ioSystem);
  virtual Primitive::Handle selectPrimitiveWithArguments(GP::System& ioSystem);
  virtual Primitive::Handle selectPrimitiveWithKArguments(unsigned int inK, GP::System& ioSystem);
  virtual Primitive::Handle selectPrimitiveWithoutArgument(GP::System& ioSystem);
  virtual void              read(XMLNode::Handle& inNode);
  virtual void              write(XMLStreamer& ioStreamer) const;

protected:

  typedef std::vector< std::pair<double,Primitive::Handle> > Roulette;

  void              computeRoulette(Roulette& outRoulette, unsigned int inNbArgs) const;
  Primitive::Handle selectWithRoulette(const Roulette& inRoulette, GP::System& ioSystem) const;
  
  Roulette                        mWeights;      //!< Weigths of primitives.
  Map                             mNames;        //!< Look-up table (by name) of the primitives.
  std::map<unsigned int,Roulette> mMap;          //!< Nb. args -> weigths map.
  std::string                     mRootType;     //!< Associated root type. Used only with STGP.

// moje:
public:
	void ChangeWeight(unsigned int index, double value);
};

}
}

#endif // Beagle_WeightedPrimitiveSet_h

