#include "beagle/GP.hpp"
#include "fitnes.hpp"
#include "AfterGenOp.h"
#include "TermMaxNoImpGenOp.hpp"
#include "WeightedPrimitiveSet.h"
#include "Pos.h"

#define _USE_MATH_DEFINES
#include <cmath>
#include <cstdlib>
#include <iostream>
#include <stdexcept>
#include <vector>
#include <numeric>

// dijelovi za DBeagle
//#include "dbeagle/EvolverGP.hpp"
//using namespace DBeagle;

using namespace std;
using namespace Beagle;

std::string path = "";	// globalna :( varijabla za path - treba je razred fitnes
PrviEvalOp::Handle EvalOp; // globalni fitnes objekt
#ifndef TREES
#ifdef ADAPTATION
GP::WeightedPrimitiveSet::Handle Set = new GP::WeightedPrimitiveSet;	// globalni skup cvorova
#else
// ovo mora biti kada se ucitava iz seed fajla! pojma nemam zasto!
GP::PrimitiveSet::Handle Set = new GP::PrimitiveSet;
#endif	// ADAPTATION
#endif	// TREES

int main(int argc, char *argv[]) {
try {
	// inicijalizacija globalnih varijabli
	EvalOp = new PrviEvalOp;

#ifdef TREES	// koristimo visestablenu strukturu
	cout << "Stabla!\n"; fflush(stdout);
	// 1: Build primitives.
    GP::PrimitiveSet::Handle lSet1 = new GP::PrimitiveSet;	// stablo odluke
	GP::PrimitiveSet::Handle lSet2 = new GP::PrimitiveSet;	// prvo pravilo
	GP::PrimitiveSet::Handle lSet3 = new GP::PrimitiveSet;	// drugo pravilo
	// pozovemo funkciju koja definira imena funkcija i terminala
	EvalOp->DefineNodeNames();
	// funkcija koja cita i odredjuje aktivne terminale iz konf. datoteke
	EvalOp->ReadTerminals();
	for(int i=OFFSET; i<OFFSET+TERMINALS; i++)
	{	if(Nodes[i].active == true)
		{	//Set->insert(new GP::TokenT<Double>(Nodes[i].name));
			lSet2->insert(new GP::TokenT<Double>(Nodes[i].name));
			lSet3->insert(new GP::TokenT<Double>(Nodes[i].name));
		}
	}
	// funkcije stabla pravila
	lSet2->insert(new GP::Add);
	lSet2->insert(new GP::Subtract);
	lSet2->insert(new GP::Multiply);
	lSet2->insert(new GP::Divide);
	lSet2->insert(new GP::Pos);
	lSet2->insert(new GP::Sqr, 0.5);
	lSet3->insert(new GP::Add);
	lSet3->insert(new GP::Subtract);
	lSet3->insert(new GP::Multiply);
	lSet3->insert(new GP::Divide);
	lSet3->insert(new GP::Pos);
	lSet3->insert(new GP::Sqr, 0.5);

	// funkcije za stablo odluke
	lSet1->insert(new GP::Add);
	lSet1->insert(new GP::Subtract);
	lSet1->insert(new GP::Multiply);
	lSet1->insert(new GP::Divide);
	lSet1->insert(new GP::Pos);
	lSet1->insert(new GP::Sqr, 0.5);
	lSet1->insert(new GP::IfGT, 0.5);
	// terminali stabla odluke
	lSet1->insert(new GP::TokenT<Double>("MNOPr"));
	lSet1->insert(new GP::TokenT<Double>("MTWK"));
	lSet1->insert(new GP::TokenT<Double>("MTWKav"));
	lSet1->insert(new GP::TokenT<Double>("MNOPw"));
	lSet1->insert(new GP::TokenT<Double>("MTWKr"));
	lSet1->insert(new GP::TokenT<Double>("MUTL"));
	// tek ovdje im postavimo aktivnost (da ne budu ukljuceni u druga stabla)
	Nodes[T_MNOPr].active = true;
	Nodes[T_MNOPw].active = true;
	Nodes[T_MTWK].active = true;
	Nodes[T_MTWKr].active = true;
	Nodes[T_MTWKav].active = true;
	Nodes[T_MUTL].active = true;

	GP::PrimitiveSuperSet::Handle lSuperSet = new GP::PrimitiveSuperSet;
	lSuperSet->insert(lSet1);
	lSuperSet->insert(lSet2);
	lSuperSet->insert(lSet3);
	// 2: Build a system.
	GP::System::Handle lSystem = new GP::System(lSuperSet);

#else	// koristimo samo jedno stablo (globalni objekt)
	cout << "Jedno stablo!\n"; fflush(stdout);
	// pozovemo funkciju koja definira imena funkcija i terminala
	EvalOp->DefineNodeNames();
	// funkcija koja cita i odredjuje aktivne terminale iz konf. datoteke
	EvalOp->ReadTerminals();
	for(int i=OFFSET; i<OFFSET+TERMINALS; i++)
	{	if(Nodes[i].active == true)
		{	Set->insert(new GP::TokenT<Double>(Nodes[i].name));
		}
	}
	Set->insert(new GP::Add);
	Set->insert(new GP::Subtract);
	Set->insert(new GP::Multiply);
	Set->insert(new GP::Divide);
	Set->insert(new GP::Pos);
	Set->insert(new GP::Sqr, 0.2);
	//Set->insert(new GP::IfGT, 0.2);
    //Set->insert(new GP::Exp);
    //Set->insert(new GP::Log);
	//Set->insert(new GP::EphemeralDouble);
    GP::System::Handle lSystem = new GP::System(Set);
#endif

    // 3: Build evaluation operator.
    //PrviEvalOp::Handle EvalOp = new PrviEvalOp;	// ne treba, imamo globalni... :(

    // 4: Build an evolver and a vivarium.
    // obicni evolver
		GP::Evolver::Handle lEvolver = new GP::Evolver(EvalOp);
	// evolver za DBeagle
		//DBeagle::EvolverGP::Handle lEvolver = new DBeagle::EvolverGP(EvalOp);
    // dodajmo vlastite operatore u Evolver
	lEvolver->addOperator(new AfterGenOp);
	lEvolver->addOperator(new TermMaxNoImpGenOp);
	GP::Vivarium::Handle lVivarium = new GP::Vivarium;
    // 5: Initialize and evolve the vivarium.
    lEvolver->initialize(lSystem, argc, argv);
    lEvolver->evolve(lVivarium);
  }
  catch(Exception& inException) {
    inException.terminate();
  }
  catch(exception& inException) {
    cerr << "Standard exception catched:" << endl;
    cerr << inException.what() << endl << flush;
    return 1;
  }
  catch(...) {
    cerr << "Unknown exception catched!" << endl << flush;
    return 1;
  }
  return 0;
}

